## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
## Script for the biogeographic analysis used in
## "Evolution of dispersal, habit, and pollination in Africa pushed Apocynaceae diversification after the Eocene-Oligocene climate transition"
## by Bitencourt et al.; written by Nicolai Nürk, Mar 2021 (nuerk[@]uni-bayreuth.de)
##
##
## The script produces
## (1) exploratory analysis & biogeographic model testing
## (2) biogeographic stochastic mapping (BSM)
## in BioGeoBEARS v1.1.2
##
##
## following Nick Matzke's example script available at:
## http://phylo.wikidot.com/biogeobears#script & http://phylo.wikidot.com/example-biogeobears-scripts
## by Nicholas J. Matzke, GPL-3 (accessed 15. Dec 2020)
##
## for instructions on installation of required packages
## see: http://phylo.wikidot.com/biogeobears#installation & https://github.com/nmatzke/BioGeoBEARS
##
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
##
## The script runs the following analyses:
##
## (i) Exploratory:
##    M1   : the default 'unconstrained' model (all dispersal multipliers = 1)
##    M1s  : the default model with selected range states (not all possible "combinations of areas" (= ranges) are allowed at internal nodes)
##
## (ii) Exploratory: stratified
##    M2   : the 'stratified' model with four time slices (TS: with varying dispersal probs among areas and time slices)
##    M2s  : the 'stratified' model with four time slices (TS) and selected range states
##
## (iii) Hypothesis tests (based on M2 and M2s model; where M2 = MX and M2s = MXs models):
##    M3/M3s   :	South America	origin ("S")
##    M4/M4s   :	Africa origin ("A")
##    M5/M5s   :	West-Gondwana origin  ("S+A")
##    M6/M6s   :	Gondwana origin 	("S+A+O")
##    M7/M7s   :	tropical Asia origin 	("T")
##    M8/M8s   :	eastern Gondwanan origin	("T+O")
##    M9/M9s   :	Laurasia origin 	("N+E")
##    M10/M10s :	tropical Laurasia origin	("N+E+T")
##    M11/M11s :	pantropical.1 origin	("S+A+T")
##    M12/M12s :	pantropical.2 origin	("S+A+O+T")
##
## (iv) Biogeographic stochastic mapping (BSM)
##    will do 1000 stochastic mappings using the *best* model (see: point v)
##    and saves results into the folder "BSM"
##
## (v) Statistics:
##    Model testing and model weighting (based on the Akaike information criterion 'AIC'; see end of this script)
##
##
## …assuming Apocynaceae species to be distributed in 6 areas
## (North America "N", South America "S", Africa "A", tropical southeast Asia "T", Oceania incl. Australasia "O", and temperate Eurasia E") 
## and
## considering in (i) and (ii) all 6 biogeobears models: DEC, DEC+j, Bayarea-like (Landis, Matzke, Moore, & Huelsenbeck, 2013), Bayarea-like+j, DIVA-like (Ronquist 1997), and DIVA-like+j, 
## in (iii) the DEC, Bayarea-like, and DIVA-like models,
## and in (iv) the best model (identified in (v) Statistics)
##
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
## 
## Files needed:
##
##  SCRIPTS
##  (1) Main:  "script_0_master_Biogeobears_Apocyn_Dist_v3_nmn_v1.R"
##      (most to set up/adjust for individual analysis is in this script)
##  (2) run (i) exploratory analysis incl. plotting: "script_1_fitModels_biogeobears_Apocynaceae.R"
##  (3) run (ii) stratified analysis incl. plotting: "script_2_fitModels_stratified_biogeobears_Apocynaceae.R"
##  (4) run (iii) 'test models' analysis incl. plotting: : "script_3_fitTestModels_biogeobears_Apocynaceae.R"
##  (5) run (iv) BSM analysis incl. plotting: : "script_4_fitBSM_on_bestModel_biogeobears_Apocynaceae.R"
##
##  DATA
##  (6) dated phylogeny: "ApocynTree_Fishbein2018_n1041_Llad_newNames.tre"
##  (7) distribution file: "Apocynaceae_allDATA_20210616_distributions.data" [with single letter code - need for BSM!!!]
##
##  SPECIFIC (biogeobears)
##  (8) time periods (TS): "timeperiods.txt"
##  (9) dispersal probabilities per period: "manual_dispersal_multipliers.txt"
##  (X) NOT RUN: range states (possible combinations of geographic areas): "states_allowed.txt" Not used here, we do 'manual modification' of range states lists (s. below: "Set and *select* STATES and RANGE lists")
##
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##



## NOTE that Biogeobears assumes your working directory is being set (check with "getwd()" and set with "setwd("/your/PATH")") and contains *all* input files! 
setwd("/Users/Nicolai/Documents/Work/Projects/Apocynaceae/Apocynaceae_BiogeoDiv_Ale/Analyses_nmn/Biogeo/4_Distribution_v3_newNames")

rm(list = ls())

## load required packages
library(GenSA)     # insta GenSA is better than optimx (although somewhat slower)
library(FD)        # for FD::maxent() (make sure this is up-to-date)
library(parallel)  # if you want to use multi-core functionality; some systems/R versions prefer library(snow), try either
library(MultinomialCI)    # For 95% CIs on BSM counts
library(rexpokit)
library(cladoRcpp)
library(BioGeoBEARS)




## Load the RESULTS
# load("allApocynaceae_BioGeoBears_results.RData")



## OR — START AGAIN analyses:
sink(file = "runtime.txt", append = FALSE)
cat("\n\nStart run: 'script_0_master_Biogeobears_Apocynaceae.R' \non iMac 2017 (OS 10.13.6, 4.2 GHz i7, 64 GB memory):\n\n")
date()
sink()
########################################
## basic settings &
## load tree and distribution file 
####################


## set the path+name to your tree & distribtion files (these are needed for Biogeoberas' specific 'model' object class, called "BioGeoBEARS_run_object")
## the phylogeny needs to be in the Newick format
trfn  <-  "/Users/Nicolai/Documents/Work/Projects/Apocynaceae/Apocynaceae_BiogeoDiv_Ale/Analyses_nmn/Biogeo/4_Distribution_v3_newNames/ApocynTree_Fishbein2018_n1041_newNames20210616.tre"
geogfn  <-  "/Users/Nicolai/Documents/Work/Projects/Apocynaceae/Apocynaceae_BiogeoDiv_Ale/Analyses_nmn/Biogeo/4_Distribution_v3_newNames/Apocynaceae_allDATA_20210616_distributions.data"


## set up dirs and copy files
## working directory
wd <- "/Users/Nicolai/Documents/Work/Projects/Apocynaceae/Apocynaceae_BiogeoDiv_Ale/Analyses_nmn/Biogeo/4_Distribution_v3_newNames"

## Make a dir for the PDF plots of results
dir.create("Results_plots")
wdp <- "/Users/Nicolai/Documents/Work/Projects/Apocynaceae/Apocynaceae_BiogeoDiv_Ale/Analyses_nmn/Biogeo/4_Distribution_v3_newNames/Results_plots"

## Make dir for BSM and copy the distribution file with area names in single letters (required for BSM)
dir.create("BSM")
#file.copy(geogfn, "BSM")
wdBSM <- "/Users/Nicolai/Documents/Work/Projects/Apocynaceae/Apocynaceae_BiogeoDiv_Ale/Analyses_nmn/Biogeo/4_Distribution_v3_newNames/BSM"

## Make sub-dir for the first BSM run and copy the distribution file with area names in single letters (required for BSM)
dir.create("BSM/BSM_M1s.dec")
file.copy(geogfn, "BSM/BSM_M1s.dec")
wdBSM1 <- "/Users/Nicolai/Documents/Work/Projects/Apocynaceae/Apocynaceae_BiogeoDiv_Ale/Analyses_nmn/Biogeo/4_Distribution_v3_newNames/BSM/BSM_M1s.dec"

## Make sub-dir for the second BSM run (+j) and copy the distribution file with area names in single letters (required for BSM)
dir.create("BSM/BSM_M1s.dec.j")
file.copy(geogfn, "BSM/BSM_M1s.dec.j")
wdBSM2 <- "/Users/Nicolai/Documents/Work/Projects/Apocynaceae/Apocynaceae_BiogeoDiv_Ale/Analyses_nmn/Biogeo/4_Distribution_v3_newNames/BSM/BSM_M1s.dec.j"





## read in the tree & distribution files (needed also for plotting)
tr <- read.tree(trfn)
tipranges  <-  getranges_from_LagrangePHYLIP(lgdata_fn = geogfn)
# names(tipranges@df)



## Set the maximum number of range states
## this cannot be larger than the number of areas specified (= length(tipranges@df))
## Here, we use the maximum range size (species distribution: combinations of areas) observed 
( max_range_size = max(rowSums(dfnums_to_numeric(tipranges@df))) )


## Check the number of range states (= number of different possible geographic ranges = combination of 'areas' (states) at nodes & tips)
## This depends on (a) the number of areas (= "numareas") and (b) max_range_size (= "maxareas").
numstates_from_numareas(numareas = length(tipranges@df), maxareas = 4, include_null_range = TRUE)


## Set the numbers of cores to run the analyses (assumes package 'parallel' to be installed)
n.cores <- parallel::detectCores()

## Set plotting of results (internal scripts needed for plotting)
scriptdir = np(system.file("extdata/a_scripts", package="BioGeoBEARS"))


## END basic settings
########################################



########################################
## Set and *select* STATES and RANGE lists
## used in (i & ii) 'selected ranges' model, and (iii) hypothesis testing, and (iv) BSM
####################

## NOTE that 'range states' are the possible combinations of areas (aka possible combinations of 'character states'); 
## in Biogeobears 'states' are in numbers (s. below) and 'ranges' are the areas and their combinations in letters.

## Get the area list
( areas <- getareas_from_tipranges_object(tipranges) )

## Get the list of states, where each state is a list of areas, counting from 0
## here with length = 57 (areas = 6 and max_range_size = 4)
length(states_list  <-  rcpp_areas_list_to_states_list(areas = areas, maxareas = max_range_size, include_null_range = TRUE) )

## ## Get the list of ranges (the range names in letters)
ranges_list  <-  NULL
for (i in 1:length(states_list)){
  if( (length(states_list[[i]]) == 1) && (is.na(states_list[[i]])) ){
    tmprange  <-  "_"
  } 
  else{
    tmprange  <-  paste(areas[states_list[[i]]+1], collapse = "")
  }
  ranges_list  <-  c(ranges_list, tmprange)
}

length(ranges_list) # range states, by default: 57



## Set the time slice (TS) specific *STATES LISTS* used in M1s and M2s (and derivatives)
## time slice = time period / time bin / stratum / epoch
## (to exclude unlikely ancestral population distributions)
##
## This means there is a different state space in each TS!
##
## NOTE that it appears to be better to assign the modified states lists at the end of model modifications, 
## i.e. after setting up the basic DEC/BAY/DIVA models (so it is done here:"script_1_fitExplModels_biogeobears_Apocynaceae.R")

## select the ranges to keep ('unconstrained' runs: M1s)
keep <- c("_", "N", "S", "A", "T", "O", "E", "NS", "NE", "SA", "ST", "SO", 
          "AT", "AO", "AE", "TO", "TE", "NAE", "NTO", "NTE", "NOE",
          "SAT", "SAO", "STO", "ATO", "ATE", "AOE", "TOE", "NTOE",
          "SATO", "ATOE")

## select the ranges to keep per TS ('stratified' runs: M2s)
keep.TS2 <- c("_", "N", "S", "A", "T", "O", "E", "NS", "NT", "NE", "SA", "ST", "SO", 
              "AT", "AO", "AE", "TO", "TE", "NAE", "NTO", "NTE", "NOE", 
              "SAT", "SAO", "STO", "ATO", "ATE", "AOE", "TOE", "NTOE", 
              "SATO", "ATOE")
keep.TS3 <- c("_", "N", "S", "A", "T", "O", "E", "NE", "SA", "ST", "SO", "AT", 
              "AO", "TO", "TE", "SAT", "SAO", "STO", "ATO", "ATE", "TOE", 
              "SATO", "ATOE")
keep.TS4 <- c("_", "N", "S", "A", "T", "O", "E", "NE", "SA", "SO", "AT", "AO", 
              "TO", "TE", "SAT", "SAO", "STO", "ATO", "SATO")

## assign
## for the ENTIRE tree (M1s)
states_list.sel <- states_list[ranges_list %in% keep] ## 30 states + null range (NA)

## assign per TS (M2s):
## STRATUM 1 (time slice 0—30 Ma)
## keep all
states_list.TS1 <- states_list[ranges_list %in% c(NA) == FALSE] # keep all 57
identical(states_list.TS1, states_list)
## STRATUM 2 (time slice 30—60 Ma)
## excl (most unlikely) Gondwana-Laurasia ranges
states_list.TS2 <- states_list[ranges_list %in% keep.TS2] ## 31 states
## STRATUM 3 (time slice 60—80 Ma)
## excl (unlikely) Gondwana-Laurasia ranges
states_list.TS3 <-  states_list[ranges_list %in% keep.TS3] ## 23 states
## STRATUM 4 (time slice 80—100 Ma)
## excl (almost all) Gondwana-Laurasia ranges
states_list.TS4 <- states_list[ranges_list %in% keep.TS4] ## 19 states





## (ii) Hypothesis testing
## (modify STRATUM 4):
## Select ranges of (possible) origin
## for STRATUM 4 (time slice 80—100 Ma)
## to obtain goodness-of-fit of models of origin (of Apocynaceae)

## NOTE that only the root node of the Apocynaceae tree falls into STRATUM 4
## so only this TS needs to be modified for M3—M12 to test on the origin of Apocynaceae


## Check the ranges list (get the range names)
ranges_list


## M3 — South America origin ("S")
keep.TS4.M3  <-  ranges_list %in% ranges_list[ !(ranges_list %in% c("_", "S"))] == FALSE
ranges_list_TS4.M3  <-  ranges_list[keep.TS4.M3]
length(ranges_list_TS4.M3)     # now 1: "S" (+ "_")

## M4 — Africa  ("A")
keep.TS4.M4  <-  ranges_list %in% ranges_list[ !(ranges_list %in% c("_", "A"))] == FALSE
ranges_list_TS4.M4  <-  ranges_list[keep.TS4.M4]
length(ranges_list_TS4.M4)     # 1 (+ "_")

## M5 — West-Gondwana  ("S" + "A")
keep.TS4.M5  <-  ranges_list %in% ranges_list[ !(ranges_list %in% c("_", "S", "A", "SA"))] == FALSE
ranges_list_TS4.M5  <-  ranges_list[keep.TS4.M5]
length(ranges_list_TS4.M5)     # 3 (+ "_")

## M6 — Gondwana  ("S" + "A" + "O")
keep.TS4.M6  <-  ranges_list %in% ranges_list[ !(ranges_list %in% c("_", "S", "A", "O", "SA", "SO", "AO", "SAO"))] == FALSE
ranges_list_TS4.M6  <-  ranges_list[keep.TS4.M6]
length(ranges_list_TS4.M6)     # 7 (+ "_")

## M7 — tropical Asia  ("T")
keep.TS4.M7  <-  ranges_list %in% ranges_list[ !(ranges_list %in% c("_", "T"))] == FALSE
ranges_list_TS4.M7  <-  ranges_list[keep.TS4.M7]
length(ranges_list_TS4.M7)     # 2 (+ "_")

## M8 — 'eastern' Gondwana  ("T" + "O)
keep.TS4.M8  <-  ranges_list %in% ranges_list[ !(ranges_list %in% c("_", "T", "O", "TO"))] == FALSE
ranges_list_TS4.M8  <-  ranges_list[keep.TS4.M8]
length(ranges_list_TS4.M8)     # 3 (+ "_")

## M9 — Palaearctic  ("N" + "E") (assuming that current species in T are of tropical origin; cf. M1 results)
keep.TS4.M9  <-  ranges_list %in% ranges_list[ !(ranges_list %in% c("_", "N", "E", "NE")) ] == FALSE
ranges_list_TS4.M9  <-  ranges_list[keep.TS4.M9]
length(ranges_list_TS4.M9)     # 3 (+ "_")

## M10 — Laurasia  ("N" + "E" + "T")
keep.TS4.M10  <-  ranges_list %in% ranges_list[ !(ranges_list %in% c("_", "N", "E", "T", "NE", "NT", "TE", "NTE")) ] == FALSE
ranges_list_TS4.M10  <-  ranges_list[keep.TS4.M10]
length(ranges_list_TS4.M10)     # 7 (+ "_")

## M11 — pantropical_1  ("S" + "A" + "T")
keep.TS4.M11  <-  ranges_list %in% ranges_list[ !(ranges_list %in% c("_", "S", "A", "T", "SA", "ST", "AT", "SAT"))] == FALSE
ranges_list_TS4.M11  <-  ranges_list[keep.TS4.M11]
length(ranges_list_TS4.M11)     # 15 (+ "_")

## M12 — pantropical_2  ("S" + "A" + "T" + "O")
keep.TS4.M12  <-  ranges_list %in% ranges_list[ !(ranges_list %in% c("_", "S", "A", "T", "O", "SA", "ST", "SO", "AT", "AO", "TO", "SAT", "SAO", "STO", "ATO", "SATO"))] == FALSE
ranges_list_TS4.M12  <-  ranges_list[keep.TS4.M12]
length(ranges_list_TS4.M12)     # 15 (+ "_")





## Make the stratum-specific states list (what we actually need!)
## M3
states_list.TS4.M3 <- states_list[keep.TS4.M3]
## M4
states_list.TS4.M4 <- states_list[keep.TS4.M4]
## M5
states_list.TS4.M5 <- states_list[keep.TS4.M5]
## M6
states_list.TS4.M6 <- states_list[keep.TS4.M6]
## M7
states_list.TS4.M7 <- states_list[keep.TS4.M7]
## M8
states_list.TS4.M8 <- states_list[keep.TS4.M8]
## M9
states_list.TS4.M9 <- states_list[keep.TS4.M9]
## M10
states_list.TS4.M10 <- states_list[keep.TS4.M10]
## M11
states_list.TS4.M11 <- states_list[keep.TS4.M11]
## M12
states_list.TS4.M12 <- states_list[keep.TS4.M12]


## END STATE SELECTION
########################################



########################################
## Set the general models (which are adjusted in the different analyses, s.below)
####################

## Initialize a *default* DEC model (the basic M1 model)

model.def  <- define_BioGeoBEARS_run(trfn = trfn, geogfn = NULL, 
                                     return_condlikes_table = TRUE, # good default setting to get ancestral states
                                     max_range_size = max_range_size,
                                     use_optimx = "GenSA", 
                                     num_cores_to_use = n.cores
                                     )
## hack back: needed for the function define_BioGeoBEARS_run() *v1.1.2*: geogfn is not reassigned, if 'geogfn' is given in the function (above); use: geogfn = NULL + manual assignment:
model.def$geogfn <- geogfn 


## load the dispersal multiplier matrix etc. from the text files (in wd) into the model object. Required for these to work!
model.def = readfiles_BioGeoBEARS_run(model.def)


## check inputs. Read the error messages if you get them!
check_BioGeoBEARS_run(model.def)






## Initialize a *stratified* DEC model (the basic M2 model with TS)

model.def.TS  <- define_BioGeoBEARS_run(trfn = trfn, geogfn = NULL, 
                                     return_condlikes_table = TRUE,
                                     max_range_size = max_range_size,
                                     use_optimx = "GenSA", 
                                     num_cores_to_use = n.cores,
                                     
                                     timesfn = "timeperiods.txt",                                        # the time periods used as time slices (s.below, M2-stratified analysis)
                                     dispersal_multipliers_fn = "manual_dispersal_multipliers.txt",      # the dispersal probabilities (s.below, M2-stratified analysis) 
                                     areas_allowed_fn = NA                                               # the allowed combination of areas at ancestral nodes (range states; works only for up to two area combinations, for more complex combinations s. below, 'states lists')
                                     )
## hack geogfn into the model (s.above)
model.def.TS$geogfn <- geogfn 


## load the dispersal multiplier matrix etc. from the text files (in wd!) into the model object. Required for these to work!
model.def.TS = readfiles_BioGeoBEARS_run(model.def.TS)


## Divide the tree up by time periods (time bin / stratum / epoch / time slice = TS) 
##
## NOTE that this sets Biogeobears to use a different state space in each TS
## i.e. irrespective of selecting states lists or not (s. above: range states — states lists)
model.def.TS = section_the_tree(inputs = model.def.TS, make_master_table = TRUE, plot_pieces = FALSE)
## The stratified tree is described in this table:
## model.def$master_table

## check
check_BioGeoBEARS_run(model.def.TS)

########################################



########################################
## Set the biogeographical stochastic mapping (BSM)
####################
## NOTE that for biogeographical stochastic mapping (BSM), 
## each area name in the areas/geography file has to be a *SINGLE LETTER*. 
## This is because various internal code in the BSM algorithm splits strings by letter to figure out what events are occurring. 
## see also: http://phylo.wikidot.com/biogeographical-stochastic-mapping-example-script#BSMsingle_letter

## how many stochastic maps to generate?
n.bsm <- 1000

## how many stochastic maps to print to pdf?
n.p.bsm <- 50

## using a stratified model?
stratified = FALSE



## BEST MODEL in Apocynaceae (using phy of Fischbein et al. 2018; with OG removed: n = 1041 tips) 
## is DEC with selected ranges (M1s.dec), or, if considered, DEC+j with selected ranges (M1s.dec.j)
## so:
## Initialize a *default* BSM model using the basic M1s model, with single-letter area code!
model.def.bsm <- model.def


## load the dispersal multiplier matrix etc. from the text files (in wd) into the model object. Required for these to work!
model.def.bsm = readfiles_BioGeoBEARS_run(model.def.bsm)

## check inputs
check_BioGeoBEARS_run(model.def.bsm)



## basic (?) BSM settings

## set a model name (histogram & ML/BSM comp)
model_name.dec = "BSM_M1s.dec"
model_name.dec.j = "BSM_M1s.dec.j"

## get the single letter code
areanames = names(getranges_from_LagrangePHYLIP(lgdata_fn = geogfn)@df)
## and the tip ranges (areas) for single letter…
tipranges.bsm <- getranges_from_LagrangePHYLIP(lgdata_fn = geogfn)

## NOTE: Because we did change the states_list from the default given the number of areas,  
## we have to manually make that change here as well: use *states_list.sel*
colors_list_for_states = get_colors_for_states_list_0based(areanames = areanames, max_range_size = max_range_size, plot_null_range = TRUE, 
                                                           states_list_0based = states_list.sel)
########################################




########################################
## RUN (all)
####################

## (i) each model on DEC, DEC+j, Bayarea-like, Bayarea-like+j, DIVA-like, and DIVA-like+j:
## Analysis: (i) Exploratory — M1: default model ('unconstrained')
## Analysis: (i) Exploratory — M1s: default model with selected range states
sink(file = "runtime.txt", append = TRUE)
cat("\n\nStart run: 'script_1_fitModels_biogeobears_Apocynaceae.R':\n\n")
date()
sink()
source("script_1_fitModels_biogeobears_Apocynaceae.R")
sink(file = "runtime.txt", append = TRUE, split = TRUE)
cat("\nEnd run 1:\n")
date()
sink()

## (ii) each model on DEC, DEC+j, Bayarea-like, Bayarea-like+j, DIVA-like, and DIVA-like+j:
## Analysis: (ii) Exploratory — M2: Stratified analyses 
## Analysis: (ii) Exploratory — M2s: stratified model with selected range states
sink(file = "runtime.txt", append = TRUE)
cat("\n\nStart run: 'script_2_fitModels_stratified_biogeobears_Apocynaceae.R':\n\n")
date()
sink()
source("script_2_fitModels_stratified_biogeobears_Apocynaceae.R")
sink(file = "runtime.txt", append = TRUE, split = TRUE)
cat("\nEnd script 2:\n")
date()
sink()

## (iii) each test model on DEC, Bayarea-like, and DIVA-like (using M2 and M2s):
sink(file = "runtime.txt", append = TRUE)
cat("\n\nStart run: 'script_3_fitTestModels_biogeobears_Apocynaceae.R':\n\n")
date()
sink()
source("script_3_fitTestModels_biogeobears_Apocynaceae.R")
sink(file = "runtime.txt", append = TRUE, split = TRUE)
cat("\nEnd script 3:\n")
date()
sink()

# (iv) BSM analysis on DEC, and DEC+j
sink(file = "runtime.txt", append = TRUE)
cat("\n\nStart run: 'script_4_fitBSM_on_bestModel_biogeobears_Apocynaceae.R':\n\n")
date()
sink()
source("script_4_fitBSM_on_bestModel_biogeobears_Apocynaceae.R")
sink(file = "runtime.txt", append = TRUE, split = TRUE)
cat("\nEnd script 4:\n")
date()
sink()


save.image("allApocynaceae_BioGeoBears_results.RData")
########################################




########################################
## Get the probabilities of each range state at each node
####################
## best model: res.M1s.dec
## In this table:
## - columns are states/ranges
## - rows are nodes, in APE order (tips, then root, then internal)
range_probabilities = as.data.frame(res.M1s.dec$ML_marginal_prob_each_state_at_branch_top_AT_node)

##  Get the node numbers in APE order:
trtable  <- prt(tr, printflag = FALSE)

## Make the node numbers the row names
## Make the range_list the column names
row.names(range_probabilities) = trtable$node
names(range_probabilities) = keep #for the MXs models (selected range space)
# names(range_probabilities) = ranges_list #for the MX models (full range space)
tail(range_probabilities)

## Save
write.table(range_probabilities, file="results_range_probabilities_M1s.dec.txt", quote=FALSE, sep="\t")



## 2nd best model: res.M1.dec
## In this table:
## - columns are states/ranges
## - rows are nodes, in APE order (tips, then root, then internal)
range_probabilities2 = as.data.frame(res.M1.dec$ML_marginal_prob_each_state_at_branch_top_AT_node)

##  Get the node numbers in APE order:
trtable2  <- prt(tr, printflag = FALSE)

## Make the node numbers the row names
## Make the range_list the column names
row.names(range_probabilities2) = trtable2$node
names(range_probabilities2) = ranges_list #for the MX models (full range space)
tail(range_probabilities2)

## Save
write.table(range_probabilities2, file="results_range_probabilities_M1.dec.txt", quote=FALSE, sep="\t")





## Plot node and tip numbers
pdf("Rplot_ApocynTree_Fishbein2018_n1041_Llad_newNames_NodeTipNumbers.pdf", width = 20, height = 100)
plot.phylo(tr, cex = 0.4, root.edge = TRUE, label.offset = 3); axisPhylo()
nodelabels(cex = 0.6, col = NULL, bg = "grey95", frame = "circle")
tiplabels(1:length(tr$tip.label), cex = 0.5, col = NULL, bg = "grey95", frame = "none", adj = -1)
dev.off()

########################################





## (v) Statistics ## ## ## ## ## ## ## ## ## ## ## 



########################################
## Model testing (using AICc and Akaike weights)
####################
## Compare *several* models (this is what we want)
##
## NB http://brianomeara.info/aic.html  :
## ~"
## AIC = -2 ( ln ( likelihood )) + 2 K
## where likelihood is the probability of the data given a model and K is the number of free parameters in the model [DF].
##
## AICc = -2 ( ln ( likelihood )) + 2 K * (n / ( n - K - 1))
## where n is the sample size […]: In model selection in phylogenetic comparative methods, sample size often refers to the number of taxa (Butler and King, 2004; O’Meara et al., 2006).
## ['Sample size' in the model selection approach is the number of data points (observed values) used to fit and select the competing models; so, sample size should always be the same for all comparisons, 
## since maximum likelihood and AIC model-selection methods are always comparing different models on the same data, not different data on the same model!]
##
## ∆AIC scores, or difference between the best model (smallest AIC) and each model (so the best model has a ∆AIC of zero)
## ∆AIC = AIC score for that model - min(AIC best model)
##
## Akaike weights represent the relative likelihood of a model. To calculate them, for each model first calculate the relative likelihood of the model: 
## relative likelihood:  'lnL.rel' = exp( -0.5 * ∆AIC score for that model). 
## The Akaike weight for a model is this value divided by the sum of these values across all models:
## 'Akaike weight' = lnL.rel / sum(all.lnL.rel)
## "


## COMPARE *all* MODELS using lists
##
## Assemble results into a list 
## (as many as one aims comparing)
res.all <- list(res.M1.bay, res.M1.bay.j, res.M1.dec, res.M1.dec.j, res.M1.diva, res.M1.diva.j,
                res.M1s.bay, res.M1s.bay.j, res.M1s.dec, res.M1s.dec.j, res.M1s.diva, res.M1s.diva.j,
                res.M2.bay, res.M2.bay.j, res.M2.dec, res.M2.dec.j, res.M2.diva, res.M2.diva.j,
                res.M2s.bay, res.M2s.bay.j, res.M2s.dec, res.M2s.dec.j, res.M2s.diva, res.M2s.diva.j,
                res.M3.bay, res.M3.dec, res.M3.diva, 
                res.M4.bay, res.M4.dec, res.M4.diva,
                res.M5.bay, res.M5.dec, res.M5.diva,
                res.M6.bay, res.M6.dec, res.M6.diva,
                res.M7.bay, res.M7.dec, res.M7.diva,
                res.M8.bay, res.M8.dec, res.M8.diva,
                res.M9.bay, res.M9.dec, res.M9.diva,
                res.M10.bay, res.M10.dec, res.M10.diva,
                res.M11.bay, res.M11.dec, res.M11.diva,
                res.M12.bay, res.M12.dec, res.M12.diva,
                res.M3s.bay, res.M3s.dec, res.M3s.diva, 
                res.M4s.bay, res.M4s.dec, res.M4s.diva,
                res.M5s.bay, res.M5s.dec, res.M5s.diva,
                res.M6s.bay, res.M6s.dec, res.M6s.diva,
                res.M7s.bay, res.M7s.dec, res.M7s.diva,
                res.M8s.bay, res.M8s.dec, res.M8s.diva,
                res.M9s.bay, res.M9s.dec, res.M9s.diva,
                res.M10s.bay, res.M10s.dec, res.M10s.diva,
                res.M11s.bay, res.M11s.dec, res.M11s.diva,
                res.M12s.bay, res.M12s.dec, res.M12s.diva)

## set names (order!)
names(res.all) <- c("M1.bay", "M1.bay.j", "M1.dec", "M1.dec.j", "M1.diva", "M1.diva.j",
                    "M1s.bay", "M1s.bay.j", "M1s.dec", "M1s.dec.j", "M1s.diva", "M1s.diva.j",
                    "M2.bay", "M2.bay.j", "M2.dec", "M2.dec.j", "M2.diva", "M2.diva.j",
                    "M2s.bay", "M2s.bay.j", "M2s.dec", "M2s.dec.j", "M2s.diva", "M2s.diva.j",
                    "M3.bay", "M3.dec", "M3.diva",
                    "M4.bay", "M4.dec", "M4.diva",
                    "M5.bay", "M5.dec", "M5.diva",
                    "M6.bay", "M6.dec", "M6.diva",
                    "M7.bay", "M7.dec", "M7.diva",
                    "M8.bay", "M8.dec", "M8.diva",
                    "M9.bay", "M9.dec", "M9.diva",
                    "M10.bay", "M10.dec", "M10.diva",
                    "M11.bay", "M11.dec", "M11.diva",
                    "M12.bay", "M12.dec", "M12.diva",
                    "M3s.bay", "M3s.dec", "M3s.diva",
                    "M4s.bay", "M4s.dec", "M4s.diva",
                    "M5s.bay", "M5s.dec", "M5s.diva",
                    "M6s.bay", "M6s.dec", "M6s.diva",
                    "M7s.bay", "M7s.dec", "M7s.diva",
                    "M8s.bay", "M8s.dec", "M8s.diva",
                    "M9s.bay", "M9s.dec", "M9s.diva",
                    "M10s.bay", "M10s.dec", "M10s.diva",
                    "M11s.bay", "M11s.dec", "M11s.diva",
                    "M12s.bay", "M12s.dec", "M12s.diva")

## set degree of freedom (number of free parameters per model):
## DEC/BAY/DIVA = 2; DEC+j/BAY+j/DIVA+j = 3
DF <- c(2, 3, 2, 3, 2, 3,
        2, 3, 2, 3, 2, 3,
        2, 3, 2, 3, 2, 3,
        2, 3, 2, 3, 2, 3, # 24!  / 25-60:
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2,
        2, 2, 2)


## set up a *results table* for all results
## and extract the log likelihoods ('LnL') per model
restable.all <- matrix(as.numeric(unlist(lapply(res.all, function(i) i[]["total_loglikelihood"]))), nrow = length(res.all), byrow = TRUE)
rownames(restable.all) <- names(res.all)
colnames(restable.all) <- "lnL"

## assign DF ('numparams')
restable.all <- cbind(restable.all, "DF" = DF)

## set column names (for the statistics to be added)
restable.all <- data.frame(restable.all, "AIC" = c(NA), "AICc" = c(NA), "deltaAICc" = c(NA), "wi" = c(NA))


## get the Akaike Information Criterion (AIC) per model
## AIC = -2 ( ln ( likelihood )) + 2 K
for(i in 1:length(restable.all[, 1])) restable.all[i, "AIC"] <- - 2 * restable.all[i, "lnL"] + 2 * restable.all[i, "DF"]

## get the sample size corrected AIC (AICc)
## AICc = -2 ( ln ( likelihood )) + 2 K * (n / ( n - K - 1))
for(i in 1:length(restable.all[, 1])) restable.all[i, "AICc"] <- - 2 * restable.all[i, "lnL"] + 2 * restable.all[i, "DF"] * (length(tr$tip.label) / (length(tr$tip.label) - restable.all[i, "DF"] -1))

## get the delta AICc (∆AICc)
## ∆AIC = AIC score for that model - min(AIC best model)
for(i in 1:length(restable.all[, 1])) restable.all[i, "deltaAICc"] <- restable.all[i, "AICc"] - min(restable.all[, "AICc"])
  
## get the Akaike weight (wi)
## wi = relative.lnL / sum(all.relative.lnL)
for(i in 1:length(restable.all[, 1])) restable.all[i, "wi"] <- exp(-0.5 * restable.all[i, "deltaAICc"]) / sum(exp(-0.5 * restable.all[, "deltaAICc"]))


## save
## round
restable.all[, 3:5] <- round(restable.all[, 3:5], 2)
## sort acc to ∆AICc
( restable.all <- restable.all[order(as.numeric(restable.all[, "deltaAICc"])), ] )
## write txt
write.table(restable.all, file = "results_ModelComparison_all_biogeobears_Apocynaceae.txt", quote = FALSE, sep = "\t")


## that's it!








# ##  ##  ## ##  ## ##  ## ##
# ##
# ## compare *selected* models (TESTING)
# ## Assemble, for example, (iii) hypothesis test results into a list (just as an example; see below line 800)
# ##
# 
# ##  ##  ## ##  ## ##  ## ##
# ##
# ## ALL (M.TS)
# 
# res.M.tests <- res.all[c("M3.bay", "M3.dec", "M3.diva",
#                          "M4.bay", "M4.dec", "M4.diva",
#                          "M5.bay", "M5.dec", "M5.diva",
#                          "M6.bay", "M6.dec", "M6.diva",
#                          "M7.bay", "M7.dec", "M7.diva",
#                          "M8.bay", "M8.dec", "M8.diva",
#                          "M9.bay", "M9.dec", "M9.diva",
#                          "M10.bay", "M10.dec", "M10.diva",
#                          "M11.bay", "M11.dec", "M11.diva",
#                          "M12.bay", "M12.dec", "M12.diva")]
# 
# ## set up a *results table* for all results
# ## and extract the log likelihoods ('LnL') per model
# restable.M.tests <- matrix(as.numeric(unlist(lapply(res.M.tests, function(i) i[]["total_loglikelihood"]))), nrow = length(res.M.tests), byrow = TRUE)
# rownames(restable.M.tests) <- names(res.M.tests)
# colnames(restable.M.tests) <- "lnL"
# 
# ## assign DF ('numparams')
# restable.M.tests <- cbind(restable.M.tests, "DF" = DF[26])
# 
# ## set  column names for the statistics to add
# restable.M.tests <- data.frame(restable.M.tests, "AIC" = c(NA), "AICc" = c(NA), "deltaAICc" = c(NA), "wi" = c(NA))
# 
# ## get the Akaike Information Criterion (AIC) per model
# ## AIC = -2 ( ln ( likelihood )) + 2 K
# for(i in 1:length(restable.M.tests[, 1])) restable.M.tests[i, "AIC"] <- - 2 * restable.M.tests[i, "lnL"] + 2 * restable.M.tests[i, "DF"]
# ## get the sample size corrected AIC (AICc)
# ## AICc = -2 ( ln ( likelihood )) + 2 K * (n / ( n - K - 1))
# for(i in 1:length(restable.M.tests[, 1])) restable.M.tests[i, "AICc"] <- - 2 * restable.M.tests[i, "lnL"] + 2 * restable.M.tests[i, "DF"] * (length(tr$tip.label) / (length(tr$tip.label) - restable.M.tests[i, "DF"] -1))
# ## get the delta AICc (∆AICc)
# ## ∆AIC = AIC score for that model - min(AIC best model)
# for(i in 1:length(restable.M.tests[, 1])) restable.M.tests[i, "deltaAICc"] <- restable.M.tests[i, "AICc"] - min(restable.M.tests[, "AICc"])
# ## get the Akaike weight (wi)
# ## wi = relative.lnL / sum(all.relative.lnL)
# for(i in 1:length(restable.M.tests[, 1])) restable.M.tests[i, "wi"] <- exp(-0.5 * restable.M.tests[i, "deltaAICc"]) / sum(exp(-0.5 * restable.M.tests[, "deltaAICc"]))
# 
# ## save statistic results
# ## rm digits
# restable.M.tests[, 3:5] <- round(restable.M.tests[, 3:5], 0)
# ## sort acc to ∆AICc
# ( restable.M.tests <- restable.M.tests[order(as.numeric(restable.M.tests[, "deltaAICc"])), ] )
# ## write txt
# write.table(restable.M.tests, file = "results_HypothesisTests_M_all_biogeobears_Apocynaceae.txt", quote = FALSE, sep = "\t")


## NB the above step can be modified so every subset of models be compared… see folder "Results_full_analyses_output" on DRYAD for full script used in the study

########################################
save.image("allApocynaceae_BioGeoBears_results.RData")


sink(file = "runtime.txt", append = TRUE, split = TRUE)
cat("\nEnd runs:\n")
date()
cat("\n\nusing\n\n")
sessionInfo()
sink()
## END





