## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
## Script for the biogeographic analysis used in
## "Evolution of dispersal, habit, and pollination in Africa pushed Apocynaceae diversification after the Eocene-Oligocene climate transition"
## by Bitencourt et al.; written by Nicolai Nürk, dec 2020 (nuerk[@]uni-bayreuth.de)
##
##
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
##  can use 'arrows' left to "#############" for structuring
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##



## Analysis: (i) Exploratory ## M1: default model (unconstrained) ## ## ## ## ## ## ## ## ## 



####################################################
## fit M1-DEC
#############

## define the M1-dec model ('unconstrained' run under the default DEC model)
M1.dec <- model.def

## check
check_BioGeoBEARS_run(M1.dec)


## run M1.dec
res.M1.dec <- bears_optim_run(M1.dec)


## plot the results
setwd(wdp)
title.plot <- "DEC on Apocynaceae M1_unconstrained"
pdf("Rplot_DEC_M1_unconstrained.pdf", width = 20, height = 100)

## States
plot_BioGeoBEARS_results(res.M1.dec, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M1.dec, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
## save env
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M1-DEC+j
#############

## define the M1-dec.j model (unconstrained run under DEC+J model)
M1.dec.j <- model.def

## Get the ML parameter values from the 2-parameter nested model
## (this will ensure that the 3-parameter model always does at least as good)
dstart = res.M1.dec$outputs@params_table["d","est"]
estart = res.M1.dec$outputs@params_table["e","est"]
jstart = 0.0001

## Input starting values for d, e
M1.dec.j$BioGeoBEARS_model_object@params_table["d","init"] = dstart
M1.dec.j$BioGeoBEARS_model_object@params_table["d","est"] = dstart
M1.dec.j$BioGeoBEARS_model_object@params_table["e","init"] = estart
M1.dec.j$BioGeoBEARS_model_object@params_table["e","est"] = estart

## Add j as a free parameter
M1.dec.j$BioGeoBEARS_model_object@params_table["j","type"] = "free"
M1.dec.j$BioGeoBEARS_model_object@params_table["j","init"] = jstart
M1.dec.j$BioGeoBEARS_model_object@params_table["j","est"] = jstart

check_BioGeoBEARS_run(M1.dec.j)



## run M1.dec.j
res.M1.dec.j <- bears_optim_run(M1.dec.j)

## plot the results
setwd(wdp)
title.plot <- "DEC+j on Apocynaceae M1_unconstrained"
pdf("Rplot_DECJ_M1_unconstrained.pdf", width = 20, height = 100)

## States
plot_BioGeoBEARS_results(res.M1.dec.j, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M1.dec.j, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M1-Bayarea-like
#############

## define the M1-bay model (unconstrained run under BAYAREALIKE model)
M1.bay <- model.def

## No subset sympatry
M1.bay$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M1.bay$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M1.bay$BioGeoBEARS_model_object@params_table["s","est"] = 0.0

## No vicariance
M1.bay$BioGeoBEARS_model_object@params_table["v","type"] = "fixed"
M1.bay$BioGeoBEARS_model_object@params_table["v","init"] = 0.0
M1.bay$BioGeoBEARS_model_object@params_table["v","est"] = 0.0

## Adjust linkage between parameters
M1.bay$BioGeoBEARS_model_object@params_table["ysv","type"] = "1-j"
M1.bay$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/1"
M1.bay$BioGeoBEARS_model_object@params_table["y","type"] = "1-j"

## Only sympatric/range-copying (y) events allowed, and with 
## exact copying (both descendants always the same size as the ancestor)
M1.bay$BioGeoBEARS_model_object@params_table["mx01y","type"] = "fixed"
M1.bay$BioGeoBEARS_model_object@params_table["mx01y","init"] = 0.9999
M1.bay$BioGeoBEARS_model_object@params_table["mx01y","est"] = 0.9999

## Check the inputs
check_BioGeoBEARS_run(M1.bay)



## run M1.bay
res.M1.bay <- bears_optim_run(M1.bay)

## plot the results
setwd(wdp)
title.plot <- "Bayarea on Apocynaceae M1_unconstrained"
pdf("Rplot_BAY_M1_unconstrained.pdf", width = 20, height = 100)
## States
plot_BioGeoBEARS_results(res.M1.bay, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M1.bay, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M1-Bayarea-like+j
#############

## define the M1-bay.j model (unconstrained run under BAYAREALIKE+J model)
M1.bay.j <- model.def

## Get the ML parameter values from the 2-parameter nested model
## (this will ensure that the 3-parameter model always does at least as good)
dstart <- res.M1.bay$outputs@params_table["d","est"]
estart <- res.M1.bay$outputs@params_table["e","est"]
jstart <- 0.0001

## Input starting values for d, e
M1.bay.j$BioGeoBEARS_model_object@params_table["d","init"] = dstart
M1.bay.j$BioGeoBEARS_model_object@params_table["d","est"] = dstart
M1.bay.j$BioGeoBEARS_model_object@params_table["e","init"] = estart
M1.bay.j$BioGeoBEARS_model_object@params_table["e","est"] = estart

## No subset sympatry
M1.bay.j$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M1.bay.j$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M1.bay.j$BioGeoBEARS_model_object@params_table["s","est"] = 0.0

## No vicariance
M1.bay.j$BioGeoBEARS_model_object@params_table["v","type"] = "fixed"
M1.bay.j$BioGeoBEARS_model_object@params_table["v","init"] = 0.0
M1.bay.j$BioGeoBEARS_model_object@params_table["v","est"] = 0.0

## *DO* allow jump dispersal/founder-event speciation (set the starting value close to 0)
M1.bay.j$BioGeoBEARS_model_object@params_table["j","type"] = "free"
M1.bay.j$BioGeoBEARS_model_object@params_table["j","init"] = jstart
M1.bay.j$BioGeoBEARS_model_object@params_table["j","est"] = jstart

## Under BAYAREALIKE+J, the max of "j" should be 1, not 3 (as is default in DEC+J) or 2 (as in DIVALIKE+J)
M1.bay.j$BioGeoBEARS_model_object@params_table["j","max"] = 0.99999

## Adjust linkage between parameters
M1.bay.j$BioGeoBEARS_model_object@params_table["ysv","type"] = "1-j"
M1.bay.j$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/1"
M1.bay.j$BioGeoBEARS_model_object@params_table["y","type"] = "1-j"

## Only sympatric/range-copying (y) events allowed, and with 
## exact copying (both descendants always the same size as the ancestor)
M1.bay.j$BioGeoBEARS_model_object@params_table["mx01y","type"] = "fixed"
M1.bay.j$BioGeoBEARS_model_object@params_table["mx01y","init"] = 0.9999
M1.bay.j$BioGeoBEARS_model_object@params_table["mx01y","est"] = 0.9999

## NOTE (NJM, 2014-04): BAYAREALIKE+J seems to crash on some computers, usually Windows 
## machines. I can't replicate this on my Mac machines, but it is almost certainly
## just some precision under-run issue, when optim/optimx tries some parameter value 
## just below zero.  The "min" and "max" options on each parameter are supposed to
## prevent this, but apparently optim/optimx sometimes go slightly beyond 
## these limits.  Anyway, if you get a crash, try raising "min" and lowering "max" 
## slightly for each parameter:
M1.bay.j$BioGeoBEARS_model_object@params_table["d","min"] = 0.0000001
M1.bay.j$BioGeoBEARS_model_object@params_table["d","max"] = 4.9999999
M1.bay.j$BioGeoBEARS_model_object@params_table["e","min"] = 0.0000001
M1.bay.j$BioGeoBEARS_model_object@params_table["e","max"] = 4.9999999
M1.bay.j$BioGeoBEARS_model_object@params_table["j","min"] = 0.00001
M1.bay.j$BioGeoBEARS_model_object@params_table["j","max"] = 0.99999

check_BioGeoBEARS_run(M1.bay.j)



## run M1.bay.j
res.M1.bay.j <- bears_optim_run(M1.bay.j)

## plot the results
setwd(wdp)
title.plot <- "bayarea+j on Apocynaceae M1_unconstrained"
pdf("Rplot_BAYJ_M1_unconstrained.pdf", width = 20, height = 100)
## States
plot_BioGeoBEARS_results(res.M1.bay.j, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M1.bay.j, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M1-DIVA
#############

## define the M1-diva model (unconstrained run under DIVALIKE model)
M1.diva <- model.def

## Remove subset-sympatry
M1.diva$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M1.diva$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M1.diva$BioGeoBEARS_model_object@params_table["s","est"] = 0.0
M1.diva$BioGeoBEARS_model_object@params_table["ysv","type"] = "2-j"
M1.diva$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/2"
M1.diva$BioGeoBEARS_model_object@params_table["y","type"] = "ysv*1/2"
M1.diva$BioGeoBEARS_model_object@params_table["v","type"] = "ysv*1/2"

## Allow classic, widespread vicariance; all events equiprobable
M1.diva$BioGeoBEARS_model_object@params_table["mx01v","type"] = "fixed"
M1.diva$BioGeoBEARS_model_object@params_table["mx01v","init"] = 0.5
M1.diva$BioGeoBEARS_model_object@params_table["mx01v","est"] = 0.5

check_BioGeoBEARS_run(M1.diva)



## run M1.diva
res.M1.diva <- bears_optim_run(M1.diva)

## plot the results
setwd(wdp)
title.plot <- "DIVA on Apocynaceae M1_unconstrained"
pdf("Rplot_DIVA_M1_unconstrained.pdf", width = 20, height = 100)

## States
plot_BioGeoBEARS_results(res.M1.diva, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M1.diva, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M1-DIVA+j
#############

## define the M1-diva model (unconstrained run under DIVALIKE+J model)
M1.diva.j <- model.def

## Get the ML parameter values from the 2-parameter nested model
## (this will ensure that the 3-parameter model always does at least as good)
dstart = res.M1.diva$outputs@params_table["d","est"]
estart = res.M1.diva$outputs@params_table["e","est"]
jstart = 0.0001

## Input starting values for d, e
M1.diva.j$BioGeoBEARS_model_object@params_table["d","init"] = dstart
M1.diva.j$BioGeoBEARS_model_object@params_table["d","est"] = dstart
M1.diva.j$BioGeoBEARS_model_object@params_table["e","init"] = estart
M1.diva.j$BioGeoBEARS_model_object@params_table["e","est"] = estart

## Remove subset-sympatry
M1.diva.j$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M1.diva.j$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M1.diva.j$BioGeoBEARS_model_object@params_table["s","est"] = 0.0
M1.diva.j$BioGeoBEARS_model_object@params_table["ysv","type"] = "2-j"
M1.diva.j$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/2"
M1.diva.j$BioGeoBEARS_model_object@params_table["y","type"] = "ysv*1/2"
M1.diva.j$BioGeoBEARS_model_object@params_table["v","type"] = "ysv*1/2"

## Allow classic, widespread vicariance; all events equiprobable
M1.diva.j$BioGeoBEARS_model_object@params_table["mx01v","type"] = "fixed"
M1.diva.j$BioGeoBEARS_model_object@params_table["mx01v","init"] = 0.5
M1.diva.j$BioGeoBEARS_model_object@params_table["mx01v","est"] = 0.5

## Add jump dispersal/founder-event speciation
M1.diva.j$BioGeoBEARS_model_object@params_table["j","type"] = "free"
M1.diva.j$BioGeoBEARS_model_object@params_table["j","init"] = jstart
M1.diva.j$BioGeoBEARS_model_object@params_table["j","est"] = jstart

## Under DIVALIKE+J, the max of "j" should be 2, not 3 (as is default in DEC+J)
M1.diva.j$BioGeoBEARS_model_object@params_table["j","min"] = 0.00001
M1.diva.j$BioGeoBEARS_model_object@params_table["j","max"] = 1.99999

check_BioGeoBEARS_run(M1.diva.j)



## run M1.diva
res.M1.diva.j <- bears_optim_run(M1.diva.j)

## plot the results
setwd(wdp)
title.plot <- "DIVA+j on Apocynaceae M1_unconstrained"
pdf("Rplot_DIVAJ_M1_unconstrained.pdf", width = 20, height = 100)

## States
plot_BioGeoBEARS_results(res.M1.diva.j, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M1.diva.j, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################




## Analysis: (i) Exploratory ## M1s: default model with selected range states ## ## ## ## ## ## ## ## ## 



####################################################
## fit M1s-DEC
#############

## define the M1-dec model (unconstrained run under DEC model with selected ranges)
M1s.dec <- model.def

## INPUT the modified states list (with selected states only) into model.def.sel
M1s.dec$states_list = states_list.sel

check_BioGeoBEARS_run(M1s.dec)


## run M1.dec
res.M1s.dec <- bears_optim_run(M1s.dec)



## plot the results
setwd(wdp)
title.plot <- "DEC on Apocynaceae M1s_unconstrained_selectedRanges"
pdf("Rplot_DEC_M1s_unconstrained_selRanges.pdf", width = 20, height = 100)

## States
plot_BioGeoBEARS_results(res.M1s.dec, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, # use 'plotlegend = TRUE,' if needed
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M1s.dec, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M1s-DEC+j
#############

## define the M1s-dec.j model (unconstrained run under DEC+J model with selected ranges)
M1s.dec.j <- model.def

## Get the ML parameter values from the 2-parameter nested model
## (this will ensure that the 3-parameter model always does at least as good)
dstart = res.M1s.dec$outputs@params_table["d","est"]
estart = res.M1s.dec$outputs@params_table["e","est"]
jstart = 0.0001

## Input starting values for d, e
M1s.dec.j$BioGeoBEARS_model_object@params_table["d","init"] = dstart
M1s.dec.j$BioGeoBEARS_model_object@params_table["d","est"] = dstart
M1s.dec.j$BioGeoBEARS_model_object@params_table["e","init"] = estart
M1s.dec.j$BioGeoBEARS_model_object@params_table["e","est"] = estart

## Add j as a free parameter
M1s.dec.j$BioGeoBEARS_model_object@params_table["j","type"] = "free"
M1s.dec.j$BioGeoBEARS_model_object@params_table["j","init"] = jstart
M1s.dec.j$BioGeoBEARS_model_object@params_table["j","est"] = jstart


## INPUT the modified states list (with selected states only) into model.def.sel
M1s.dec.j$states_list = states_list.sel

check_BioGeoBEARS_run(M1s.dec.j)



## run M1s.dec.j
res.M1s.dec.j <- bears_optim_run(M1s.dec.j)

## plot the results
setwd(wdp)
title.plot <- "DEC+j on Apocynaceae M1s_unconstrained_selectedRanges"
pdf("Rplot_DECJ_M1s_unconstrained_selRanges.pdf", width = 20, height = 100)

## States
plot_BioGeoBEARS_results(res.M1s.dec.j, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M1s.dec.j, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M1s-Bayarea-like
#############

## define the M1s-bay model (unconstrained run under BAYAREALIKE model with selected ranges)
M1s.bay <- model.def

## No subset sympatry
M1s.bay$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M1s.bay$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M1s.bay$BioGeoBEARS_model_object@params_table["s","est"] = 0.0

## No vicariance
M1s.bay$BioGeoBEARS_model_object@params_table["v","type"] = "fixed"
M1s.bay$BioGeoBEARS_model_object@params_table["v","init"] = 0.0
M1s.bay$BioGeoBEARS_model_object@params_table["v","est"] = 0.0

## Adjust linkage between parameters
M1s.bay$BioGeoBEARS_model_object@params_table["ysv","type"] = "1-j"
M1s.bay$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/1"
M1s.bay$BioGeoBEARS_model_object@params_table["y","type"] = "1-j"

## Only sympatric/range-copying (y) events allowed, and with 
## exact copying (both descendants always the same size as the ancestor)
M1s.bay$BioGeoBEARS_model_object@params_table["mx01y","type"] = "fixed"
M1s.bay$BioGeoBEARS_model_object@params_table["mx01y","init"] = 0.9999
M1s.bay$BioGeoBEARS_model_object@params_table["mx01y","est"] = 0.9999


## INPUT the modified states list (with selected states only) into model.def.sel
M1s.bay$states_list = states_list.sel


## Check the inputs
check_BioGeoBEARS_run(M1s.bay)



## run M1s.bay
res.M1s.bay <- bears_optim_run(M1s.bay)

## plot the results
setwd(wdp)
title.plot <- "Bayarea on Apocynaceae M1s_unconstrained_selectedRanges"
pdf("Rplot_BAY_M1s_unconstrained_selRanges.pdf", width = 20, height = 100)
## States
plot_BioGeoBEARS_results(res.M1s.bay, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M1s.bay, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M1s-Bayarea-like+j
#############

## define the M1s-bay.j model (unconstrained run under BAYAREALIKE+J model with selected ranges)
M1s.bay.j <- model.def

## Get the ML parameter values from the 2-parameter nested model
## (this will ensure that the 3-parameter model always does at least as good)
dstart <- res.M1s.bay$outputs@params_table["d","est"]
estart <- res.M1s.bay$outputs@params_table["e","est"]
jstart <- 0.0001

## Input starting values for d, e
M1s.bay.j$BioGeoBEARS_model_object@params_table["d","init"] = dstart
M1s.bay.j$BioGeoBEARS_model_object@params_table["d","est"] = dstart
M1s.bay.j$BioGeoBEARS_model_object@params_table["e","init"] = estart
M1s.bay.j$BioGeoBEARS_model_object@params_table["e","est"] = estart

## No subset sympatry
M1s.bay.j$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M1s.bay.j$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M1s.bay.j$BioGeoBEARS_model_object@params_table["s","est"] = 0.0

## No vicariance
M1s.bay.j$BioGeoBEARS_model_object@params_table["v","type"] = "fixed"
M1s.bay.j$BioGeoBEARS_model_object@params_table["v","init"] = 0.0
M1s.bay.j$BioGeoBEARS_model_object@params_table["v","est"] = 0.0

## *DO* allow jump dispersal/founder-event speciation (set the starting value close to 0)
M1s.bay.j$BioGeoBEARS_model_object@params_table["j","type"] = "free"
M1s.bay.j$BioGeoBEARS_model_object@params_table["j","init"] = jstart
M1s.bay.j$BioGeoBEARS_model_object@params_table["j","est"] = jstart

## Under BAYAREALIKE+J, the max of "j" should be 1, not 3 (as is default in DEC+J) or 2 (as in DIVALIKE+J)
M1s.bay.j$BioGeoBEARS_model_object@params_table["j","max"] = 0.99999

## Adjust linkage between parameters
M1s.bay.j$BioGeoBEARS_model_object@params_table["ysv","type"] = "1-j"
M1s.bay.j$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/1"
M1s.bay.j$BioGeoBEARS_model_object@params_table["y","type"] = "1-j"

## Only sympatric/range-copying (y) events allowed, and with 
## exact copying (both descendants always the same size as the ancestor)
M1s.bay.j$BioGeoBEARS_model_object@params_table["mx01y","type"] = "fixed"
M1s.bay.j$BioGeoBEARS_model_object@params_table["mx01y","init"] = 0.9999
M1s.bay.j$BioGeoBEARS_model_object@params_table["mx01y","est"] = 0.9999

## so, no precision under-run issue
M1s.bay.j$BioGeoBEARS_model_object@params_table["d","min"] = 0.0000001
M1s.bay.j$BioGeoBEARS_model_object@params_table["d","max"] = 4.9999999
M1s.bay.j$BioGeoBEARS_model_object@params_table["e","min"] = 0.0000001
M1s.bay.j$BioGeoBEARS_model_object@params_table["e","max"] = 4.9999999
M1s.bay.j$BioGeoBEARS_model_object@params_table["j","min"] = 0.00001
M1s.bay.j$BioGeoBEARS_model_object@params_table["j","max"] = 0.99999


## INPUT the modified states list (with selected states only) into model.def.sel
M1s.bay.j$states_list = states_list.sel


check_BioGeoBEARS_run(M1s.bay.j)



## run M1s.bay.j
res.M1s.bay.j <- bears_optim_run(M1s.bay.j)

## plot the results
setwd(wdp)
title.plot <- "bayarea+j on Apocynaceae M1s_unconstrained_selectedRanges"
pdf("Rplot_BAYJ_M1s_unconstrained_selRanges.pdf", width = 20, height = 100)
## States
plot_BioGeoBEARS_results(res.M1s.bay.j, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M1s.bay.j, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M1s-DIVA
#############

## define the M1s-diva model (unconstrained run under DIVALIKE model with selected ranges)
M1s.diva <- model.def

## Remove subset-sympatry
M1s.diva$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M1s.diva$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M1s.diva$BioGeoBEARS_model_object@params_table["s","est"] = 0.0
M1s.diva$BioGeoBEARS_model_object@params_table["ysv","type"] = "2-j"
M1s.diva$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/2"
M1s.diva$BioGeoBEARS_model_object@params_table["y","type"] = "ysv*1/2"
M1s.diva$BioGeoBEARS_model_object@params_table["v","type"] = "ysv*1/2"

## Allow classic, widespread vicariance; all events equiprobable
M1s.diva$BioGeoBEARS_model_object@params_table["mx01v","type"] = "fixed"
M1s.diva$BioGeoBEARS_model_object@params_table["mx01v","init"] = 0.5
M1s.diva$BioGeoBEARS_model_object@params_table["mx01v","est"] = 0.5


## INPUT the modified states list (with selected states only) into model.def.sel
M1s.diva$states_list = states_list.sel

check_BioGeoBEARS_run(M1s.diva)



## run M1s.diva
res.M1s.diva <- bears_optim_run(M1s.diva)

## plot the results
setwd(wdp)
title.plot <- "DIVA on Apocynaceae M1s_unconstrained_selectedRanges"
pdf("Rplot_DIVA_M1s_unconstrained_selRanges.pdf", width = 20, height = 100)

## States
plot_BioGeoBEARS_results(res.M1s.diva, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M1s.diva, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M1s-DIVA+j
#############

## define the M1s-diva model (unconstrained run under DIVALIKE+J model with selected ranges)
M1s.diva.j <- model.def

## Get the ML parameter values from the 2-parameter nested model
## (this will ensure that the 3-parameter model always does at least as good)
dstart = res.M1s.diva$outputs@params_table["d","est"]
estart = res.M1s.diva$outputs@params_table["e","est"]
jstart = 0.0001

## Input starting values for d, e
M1s.diva.j$BioGeoBEARS_model_object@params_table["d","init"] = dstart
M1s.diva.j$BioGeoBEARS_model_object@params_table["d","est"] = dstart
M1s.diva.j$BioGeoBEARS_model_object@params_table["e","init"] = estart
M1s.diva.j$BioGeoBEARS_model_object@params_table["e","est"] = estart

## Remove subset-sympatry
M1s.diva.j$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M1s.diva.j$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M1s.diva.j$BioGeoBEARS_model_object@params_table["s","est"] = 0.0
M1s.diva.j$BioGeoBEARS_model_object@params_table["ysv","type"] = "2-j"
M1s.diva.j$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/2"
M1s.diva.j$BioGeoBEARS_model_object@params_table["y","type"] = "ysv*1/2"
M1s.diva.j$BioGeoBEARS_model_object@params_table["v","type"] = "ysv*1/2"

## Allow classic, widespread vicariance; all events equiprobable
M1s.diva.j$BioGeoBEARS_model_object@params_table["mx01v","type"] = "fixed"
M1s.diva.j$BioGeoBEARS_model_object@params_table["mx01v","init"] = 0.5
M1s.diva.j$BioGeoBEARS_model_object@params_table["mx01v","est"] = 0.5

## Add jump dispersal/founder-event speciation
M1s.diva.j$BioGeoBEARS_model_object@params_table["j","type"] = "free"
M1s.diva.j$BioGeoBEARS_model_object@params_table["j","init"] = jstart
M1s.diva.j$BioGeoBEARS_model_object@params_table["j","est"] = jstart

## Under DIVALIKE+J, the max of "j" should be 2, not 3 (as is default in DEC+J)
M1s.diva.j$BioGeoBEARS_model_object@params_table["j","min"] = 0.00001
M1s.diva.j$BioGeoBEARS_model_object@params_table["j","max"] = 1.99999


## INPUT the modified states list (with selected states only) into model.def.sel
M1s.diva.j$states_list = states_list.sel

check_BioGeoBEARS_run(M1s.diva.j)



## run M1s.diva
res.M1s.diva.j <- bears_optim_run(M1s.diva.j)

## plot the results
setwd(wdp)
title.plot <- "DIVA+j on Apocynaceae M1s_unconstrained_selectedRanges"
pdf("Rplot_DIVAJ_M1s_unconstrained_selRanges.pdf", width = 20, height = 100)

## States
plot_BioGeoBEARS_results(res.M1s.diva.j, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M1s.diva.j, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################

## END RUN script 1: fit exploratory models