## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
## Script for the biogeographic analysis used in
## "Evolution of dispersal, habit, and pollination in Africa pushed Apocynaceae diversification after the Eocene-Oligocene climate transition"
## by Bitencourt et al.; written by Nicolai Nürk, Jan 2021 (nuerk[@]uni-bayreuth.de)
##
##
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
##  can use 'arrows' left to "#############" for structure
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##



## Analysis: (ii) Exploratory/Test ## M2: Stratified analyses ## ## ## ## ## ## ## ## ## 



####################################################
## fit M2-DEC
#############


## define the M2-dec model (stratified run under DEC model)
M2.dec <- model.def.TS

check_BioGeoBEARS_run(M2.dec)



## run M2.dec
res.M2.dec <- bears_optim_run(M2.dec)

## plot the results
setwd(wdp)
title.plot <- "DEC on Apocynaceae M2_stratified"
pdf("Rplot_DEC_M2_stratified.pdf", width = 20, height = 100)

## States
plot_BioGeoBEARS_results(res.M2.dec, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M2.dec, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M2-DEC+j
#############

## define the M2-dec.j model (stratified run under DEC+J model)
M2.dec.j <- model.def.TS


## Get the ML parameter values from the 2-parameter nested model
# (this will ensure that the 3-parameter model always does at least as good)
dstart = res.M2.dec$outputs@params_table["d","est"]
estart = res.M2.dec$outputs@params_table["e","est"]
jstart = 0.0001

# Input starting values for d, e
M2.dec.j$BioGeoBEARS_model_object@params_table["d","init"] = dstart
M2.dec.j$BioGeoBEARS_model_object@params_table["d","est"] = dstart
M2.dec.j$BioGeoBEARS_model_object@params_table["e","init"] = estart
M2.dec.j$BioGeoBEARS_model_object@params_table["e","est"] = estart

# Add j as a free parameter
M2.dec.j$BioGeoBEARS_model_object@params_table["j","type"] = "free"
M2.dec.j$BioGeoBEARS_model_object@params_table["j","init"] = jstart
M2.dec.j$BioGeoBEARS_model_object@params_table["j","est"] = jstart

check_BioGeoBEARS_run(M2.dec.j)



## run M2.dec.j
res.M2.dec.j <- bears_optim_run(M2.dec.j)

## plot the results
setwd(wdp)
title.plot <- "DEC+j on Apocynaceae M2_stratified"
pdf("Rplot_DECJ_M2_stratified.pdf", width = 20, height = 100)

# States
plot_BioGeoBEARS_results(res.M2.dec.j, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
# Pie chart
plot_BioGeoBEARS_results(res.M2.dec.j, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M2-Bayarea-like
#############

## define the M2-bay model (stratified run under BAYAREALIKE model)
M2.bay <- model.def.TS

## No subset sympatry
M2.bay$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M2.bay$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M2.bay$BioGeoBEARS_model_object@params_table["s","est"] = 0.0

## No vicariance
M2.bay$BioGeoBEARS_model_object@params_table["v","type"] = "fixed"
M2.bay$BioGeoBEARS_model_object@params_table["v","init"] = 0.0
M2.bay$BioGeoBEARS_model_object@params_table["v","est"] = 0.0

## Adjust linkage between parameters
M2.bay$BioGeoBEARS_model_object@params_table["ysv","type"] = "1-j"
M2.bay$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/1"
M2.bay$BioGeoBEARS_model_object@params_table["y","type"] = "1-j"

## Only sympatric/range-copying (y) events allowed, and with 
## exact copying (both descendants always the same size as the ancestor)
M2.bay$BioGeoBEARS_model_object@params_table["mx01y","type"] = "fixed"
M2.bay$BioGeoBEARS_model_object@params_table["mx01y","init"] = 0.9999
M2.bay$BioGeoBEARS_model_object@params_table["mx01y","est"] = 0.9999

## Check the inputs
check_BioGeoBEARS_run(M2.bay)



## run M2.bay
res.M2.bay <- bears_optim_run(M2.bay)

## plot the results
setwd(wdp)
title.plot <- "Bayarea on Apocynaceae M2_stratified"
pdf("Rplot_BAY_M2_stratified.pdf", width = 20, height = 100)
## States
plot_BioGeoBEARS_results(res.M2.bay, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M2.bay, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M2-Bayarea-like+j
#############

## define the M2-bay.j model (stratified run under BAYAREALIKE+J model)
M2.bay.j <- model.def.TS

## Get the ML parameter values from the 2-parameter nested model
## (this will ensure that the 3-parameter model always does at least as good)
dstart = res.M2.bay$outputs@params_table["d","est"]
estart = res.M2.bay$outputs@params_table["e","est"]
jstart = 0.0001

## Input starting values for d, e
M2.bay.j$BioGeoBEARS_model_object@params_table["d","init"] = dstart
M2.bay.j$BioGeoBEARS_model_object@params_table["d","est"] = dstart
M2.bay.j$BioGeoBEARS_model_object@params_table["e","init"] = estart
M2.bay.j$BioGeoBEARS_model_object@params_table["e","est"] = estart

## No subset sympatry
M2.bay.j$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M2.bay.j$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M2.bay.j$BioGeoBEARS_model_object@params_table["s","est"] = 0.0

## No vicariance
M2.bay.j$BioGeoBEARS_model_object@params_table["v","type"] = "fixed"
M2.bay.j$BioGeoBEARS_model_object@params_table["v","init"] = 0.0
M2.bay.j$BioGeoBEARS_model_object@params_table["v","est"] = 0.0

## *DO* allow jump dispersal/founder-event speciation (set the starting value close to 0)
M2.bay.j$BioGeoBEARS_model_object@params_table["j","type"] = "free"
M2.bay.j$BioGeoBEARS_model_object@params_table["j","init"] = jstart
M2.bay.j$BioGeoBEARS_model_object@params_table["j","est"] = jstart

## Under BAYAREALIKE+J, the max of "j" should be 1, not 3 (as is default in DEC+J) or 2 (as in DIVALIKE+J)
M2.bay.j$BioGeoBEARS_model_object@params_table["j","max"] = 0.99999

## Adjust linkage between parameters
M2.bay.j$BioGeoBEARS_model_object@params_table["ysv","type"] = "1-j"
M2.bay.j$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/1"
M2.bay.j$BioGeoBEARS_model_object@params_table["y","type"] = "1-j"

## Only sympatric/range-copying (y) events allowed, and with 
## exact copying (both descendants always the same size as the ancestor)
M2.bay.j$BioGeoBEARS_model_object@params_table["mx01y","type"] = "fixed"
M2.bay.j$BioGeoBEARS_model_object@params_table["mx01y","init"] = 0.9999
M2.bay.j$BioGeoBEARS_model_object@params_table["mx01y","est"] = 0.9999

## so, no precision under-run issue
M2.bay.j$BioGeoBEARS_model_object@params_table["d","min"] = 0.0000001
M2.bay.j$BioGeoBEARS_model_object@params_table["d","max"] = 4.9999999
M2.bay.j$BioGeoBEARS_model_object@params_table["e","min"] = 0.0000001
M2.bay.j$BioGeoBEARS_model_object@params_table["e","max"] = 4.9999999
M2.bay.j$BioGeoBEARS_model_object@params_table["j","min"] = 0.00001
M2.bay.j$BioGeoBEARS_model_object@params_table["j","max"] = 0.99999

check_BioGeoBEARS_run(M2.bay.j)



## run M2.bay.j
res.M2.bay.j <- bears_optim_run(M2.bay.j)

## plot the results
setwd(wdp)
title.plot <- "bay.jarea on Apocynaceae M2_stratified"
pdf("Rplot_BAYJ_M2_stratified.pdf", width = 20, height = 100)
## States
plot_BioGeoBEARS_results(res.M2.bay.j, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M2.bay.j, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M2-DIVA
#############

## define the M2-diva model (stratified run under DIVALIKE model)
M2.diva <- model.def.TS

## Remove subset-sympatry
M2.diva$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M2.diva$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M2.diva$BioGeoBEARS_model_object@params_table["s","est"] = 0.0
M2.diva$BioGeoBEARS_model_object@params_table["ysv","type"] = "2-j"
M2.diva$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/2"
M2.diva$BioGeoBEARS_model_object@params_table["y","type"] = "ysv*1/2"
M2.diva$BioGeoBEARS_model_object@params_table["v","type"] = "ysv*1/2"

## Allow classic, widespread vicariance; all events equiprobable
M2.diva$BioGeoBEARS_model_object@params_table["mx01v","type"] = "fixed"
M2.diva$BioGeoBEARS_model_object@params_table["mx01v","init"] = 0.5
M2.diva$BioGeoBEARS_model_object@params_table["mx01v","est"] = 0.5

check_BioGeoBEARS_run(M2.diva)



## run M2.diva
res.M2.diva <- bears_optim_run(M2.diva)

## plot the results
setwd(wdp)
title.plot <- "DIVA on Apocynaceae M2_stratified"
pdf("Rplot_DIVA_M2_stratified.pdf", width = 20, height = 100)

## States
plot_BioGeoBEARS_results(res.M2.diva, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M2.diva, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M2-DIVA+j
#############

## define the M2-diva model (stratified run under DIVALIKE+J model)
M2.diva.j <- model.def.TS

## Get the ML parameter values from the 2-parameter nested model
## (this will ensure that the 3-parameter model always does at least as good)
dstart = res.M2.diva$outputs@params_table["d","est"]
estart = res.M2.diva$outputs@params_table["e","est"]
jstart = 0.0001

## Input starting values for d, e
M2.diva.j$BioGeoBEARS_model_object@params_table["d","init"] = dstart
M2.diva.j$BioGeoBEARS_model_object@params_table["d","est"] = dstart
M2.diva.j$BioGeoBEARS_model_object@params_table["e","init"] = estart
M2.diva.j$BioGeoBEARS_model_object@params_table["e","est"] = estart

## Remove subset-sympatry
M2.diva.j$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M2.diva.j$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M2.diva.j$BioGeoBEARS_model_object@params_table["s","est"] = 0.0
M2.diva.j$BioGeoBEARS_model_object@params_table["ysv","type"] = "2-j"
M2.diva.j$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/2"
M2.diva.j$BioGeoBEARS_model_object@params_table["y","type"] = "ysv*1/2"
M2.diva.j$BioGeoBEARS_model_object@params_table["v","type"] = "ysv*1/2"

## Allow classic, widespread vicariance; all events equiprobable
M2.diva.j$BioGeoBEARS_model_object@params_table["mx01v","type"] = "fixed"
M2.diva.j$BioGeoBEARS_model_object@params_table["mx01v","init"] = 0.5
M2.diva.j$BioGeoBEARS_model_object@params_table["mx01v","est"] = 0.5

## Add jump dispersal/founder-event speciation
M2.diva.j$BioGeoBEARS_model_object@params_table["j","type"] = "free"
M2.diva.j$BioGeoBEARS_model_object@params_table["j","init"] = jstart
M2.diva.j$BioGeoBEARS_model_object@params_table["j","est"] = jstart

## Under DIVALIKE+J, the max of "j" should be 2, not 3 (as is default in DEC+J)
M2.diva.j$BioGeoBEARS_model_object@params_table["j","min"] = 0.00001
M2.diva.j$BioGeoBEARS_model_object@params_table["j","max"] = 1.99999

check_BioGeoBEARS_run(M2.diva.j)



## run M2.diva.j
res.M2.diva.j <- bears_optim_run(M2.diva.j)

## plot the results
setwd(wdp)
title.plot <- "DIVA+j on Apocynaceae M2_stratified"
pdf("Rplot_DIVAJ_M2_stratified.pdf", width = 20, height = 100)

## States
plot_BioGeoBEARS_results(res.M2.diva.j, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M2.diva.j, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################




## Analysis: (i) Exploratory ## M2s: stratified model with selected range states ## ## ## ## ## ## ## ## ## 



####################################################
## fit M2s-DEC
#############

## define the M2s.dec model (stratified run under DEC model with selected ranges)
M2s.dec <- model.def.TS


## INPUT the modified states list (with selected states only) into model.def.TSs
M2s.dec$lists_of_states_lists_0based[[1]] = states_list.TS1
M2s.dec$lists_of_states_lists_0based[[2]] = states_list.TS2
M2s.dec$lists_of_states_lists_0based[[3]] = states_list.TS3
M2s.dec$lists_of_states_lists_0based[[4]] = states_list.TS4

check_BioGeoBEARS_run(M2s.dec)



## run M2s.dec
res.M2s.dec <- bears_optim_run(M2s.dec)

## plot the results
setwd(wdp)
title.plot <- "DEC on Apocynaceae M2s_stratified_selectedRanges"
pdf("Rplot_DEC_M2s_stratified_selRanges.pdf", width = 20, height = 100)

## States
plot_BioGeoBEARS_results(res.M2s.dec, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M2s.dec, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M2s-DEC+j
#############

## define the M2s-dec.j model (stratified run under DEC+J model with selected range states)
M2s.dec.j <- model.def.TS

## Get the ML parameter values from the 2-parameter nested model
# (this will ensure that the 3-parameter model always does at least as good)
dstart = res.M2s.dec$outputs@params_table["d","est"]
estart = res.M2s.dec$outputs@params_table["e","est"]
jstart = 0.0001

# Input starting values for d, e
M2s.dec.j$BioGeoBEARS_model_object@params_table["d","init"] = dstart
M2s.dec.j$BioGeoBEARS_model_object@params_table["d","est"] = dstart
M2s.dec.j$BioGeoBEARS_model_object@params_table["e","init"] = estart
M2s.dec.j$BioGeoBEARS_model_object@params_table["e","est"] = estart

# Add j as a free parameter
M2s.dec.j$BioGeoBEARS_model_object@params_table["j","type"] = "free"
M2s.dec.j$BioGeoBEARS_model_object@params_table["j","init"] = jstart
M2s.dec.j$BioGeoBEARS_model_object@params_table["j","est"] = jstart


## INPUT the modified states list (with selected states only) into model.def.TSs
M2s.dec.j$lists_of_states_lists_0based[[1]] = states_list.TS1
M2s.dec.j$lists_of_states_lists_0based[[2]] = states_list.TS2
M2s.dec.j$lists_of_states_lists_0based[[3]] = states_list.TS3
M2s.dec.j$lists_of_states_lists_0based[[4]] = states_list.TS4

check_BioGeoBEARS_run(M2s.dec.j)



## run M2s.dec.j
res.M2s.dec.j <- bears_optim_run(M2s.dec.j)

## plot the results
setwd(wdp)
title.plot <- "DEC+j on Apocynaceae M2s_stratified_selectedRanges"
pdf("Rplot_DECJ_M2s_stratified_selRanges.pdf", width = 20, height = 100)

# States
plot_BioGeoBEARS_results(res.M2s.dec.j, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
# Pie chart
plot_BioGeoBEARS_results(res.M2s.dec.j, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M2s-Bayarea-like
#############

## define the M2s-bay model (stratified run under BAYAREALIKE model with selected range states)
M2s.bay <- model.def.TS

## No subset sympatry
M2s.bay$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M2s.bay$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M2s.bay$BioGeoBEARS_model_object@params_table["s","est"] = 0.0

## No vicariance
M2s.bay$BioGeoBEARS_model_object@params_table["v","type"] = "fixed"
M2s.bay$BioGeoBEARS_model_object@params_table["v","init"] = 0.0
M2s.bay$BioGeoBEARS_model_object@params_table["v","est"] = 0.0

## Adjust linkage between parameters
M2s.bay$BioGeoBEARS_model_object@params_table["ysv","type"] = "1-j"
M2s.bay$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/1"
M2s.bay$BioGeoBEARS_model_object@params_table["y","type"] = "1-j"

## Only sympatric/range-copying (y) events allowed, and with 
## exact copying (both descendants always the same size as the ancestor)
M2s.bay$BioGeoBEARS_model_object@params_table["mx01y","type"] = "fixed"
M2s.bay$BioGeoBEARS_model_object@params_table["mx01y","init"] = 0.9999
M2s.bay$BioGeoBEARS_model_object@params_table["mx01y","est"] = 0.9999


## INPUT the modified states list (with selected states only) into model.def.TSs
M2s.bay$lists_of_states_lists_0based[[1]] = states_list.TS1
M2s.bay$lists_of_states_lists_0based[[2]] = states_list.TS2
M2s.bay$lists_of_states_lists_0based[[3]] = states_list.TS3
M2s.bay$lists_of_states_lists_0based[[4]] = states_list.TS4

check_BioGeoBEARS_run(M2s.bay)



## run M2s.bay
res.M2s.bay <- bears_optim_run(M2s.bay)

## plot the results
setwd(wdp)
title.plot <- "Bayarea on Apocynaceae M2s_stratified_selectedRanges"
pdf("Rplot_BAY_M2s_stratified_selRanges.pdf", width = 20, height = 100)
## States
plot_BioGeoBEARS_results(res.M2s.bay, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M2s.bay, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M2s-Bayarea-like+j
#############

## define the M2s-bay.j model (stratified run under BAYAREALIKE+J model with selected range states)
M2s.bay.j <- model.def.TS

## Get the ML parameter values from the 2-parameter nested model
## (this will ensure that the 3-parameter model always does at least as good)
dstart = res.M2s.bay$outputs@params_table["d","est"]
estart = res.M2s.bay$outputs@params_table["e","est"]
jstart = 0.0001

## Input starting values for d, e
M2s.bay.j$BioGeoBEARS_model_object@params_table["d","init"] = dstart
M2s.bay.j$BioGeoBEARS_model_object@params_table["d","est"] = dstart
M2s.bay.j$BioGeoBEARS_model_object@params_table["e","init"] = estart
M2s.bay.j$BioGeoBEARS_model_object@params_table["e","est"] = estart

## No subset sympatry
M2s.bay.j$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M2s.bay.j$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M2s.bay.j$BioGeoBEARS_model_object@params_table["s","est"] = 0.0

## No vicariance
M2s.bay.j$BioGeoBEARS_model_object@params_table["v","type"] = "fixed"
M2s.bay.j$BioGeoBEARS_model_object@params_table["v","init"] = 0.0
M2s.bay.j$BioGeoBEARS_model_object@params_table["v","est"] = 0.0

## *DO* allow jump dispersal/founder-event speciation (set the starting value close to 0)
M2s.bay.j$BioGeoBEARS_model_object@params_table["j","type"] = "free"
M2s.bay.j$BioGeoBEARS_model_object@params_table["j","init"] = jstart
M2s.bay.j$BioGeoBEARS_model_object@params_table["j","est"] = jstart

## Under BAYAREALIKE+J, the max of "j" should be 1, not 3 (as is default in DEC+J) or 2 (as in DIVALIKE+J)
M2s.bay.j$BioGeoBEARS_model_object@params_table["j","max"] = 0.99999

## Adjust linkage between parameters
M2s.bay.j$BioGeoBEARS_model_object@params_table["ysv","type"] = "1-j"
M2s.bay.j$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/1"
M2s.bay.j$BioGeoBEARS_model_object@params_table["y","type"] = "1-j"

## Only sympatric/range-copying (y) events allowed, and with 
## exact copying (both descendants always the same size as the ancestor)
M2s.bay.j$BioGeoBEARS_model_object@params_table["mx01y","type"] = "fixed"
M2s.bay.j$BioGeoBEARS_model_object@params_table["mx01y","init"] = 0.9999
M2s.bay.j$BioGeoBEARS_model_object@params_table["mx01y","est"] = 0.9999

## so, no precision under-run issue
M2s.bay.j$BioGeoBEARS_model_object@params_table["d","min"] = 0.0000001
M2s.bay.j$BioGeoBEARS_model_object@params_table["d","max"] = 4.9999999
M2s.bay.j$BioGeoBEARS_model_object@params_table["e","min"] = 0.0000001
M2s.bay.j$BioGeoBEARS_model_object@params_table["e","max"] = 4.9999999
M2s.bay.j$BioGeoBEARS_model_object@params_table["j","min"] = 0.00001
M2s.bay.j$BioGeoBEARS_model_object@params_table["j","max"] = 0.99999


## INPUT the modified states list (with selected states only) into model.def.TSs
M2s.bay.j$lists_of_states_lists_0based[[1]] = states_list.TS1
M2s.bay.j$lists_of_states_lists_0based[[2]] = states_list.TS2
M2s.bay.j$lists_of_states_lists_0based[[3]] = states_list.TS3
M2s.bay.j$lists_of_states_lists_0based[[4]] = states_list.TS4

check_BioGeoBEARS_run(M2s.bay.j)



## run M2s.bay.j
res.M2s.bay.j <- bears_optim_run(M2s.bay.j)

## plot the results
setwd(wdp)
title.plot <- "bay.jarea on Apocynaceae M2s_stratified_selectedRanges"
pdf("Rplot_BAYJ_M2s_stratified_selRanges.pdf", width = 20, height = 100)
## States
plot_BioGeoBEARS_results(res.M2s.bay.j, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M2s.bay.j, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M2s-DIVA
#############

## define the M2s-diva model (stratified run under DIVALIKE model with selected range states)
M2s.diva <- model.def.TS

## Remove subset-sympatry
M2s.diva$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M2s.diva$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M2s.diva$BioGeoBEARS_model_object@params_table["s","est"] = 0.0
M2s.diva$BioGeoBEARS_model_object@params_table["ysv","type"] = "2-j"
M2s.diva$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/2"
M2s.diva$BioGeoBEARS_model_object@params_table["y","type"] = "ysv*1/2"
M2s.diva$BioGeoBEARS_model_object@params_table["v","type"] = "ysv*1/2"

## Allow classic, widespread vicariance; all events equiprobable
M2s.diva$BioGeoBEARS_model_object@params_table["mx01v","type"] = "fixed"
M2s.diva$BioGeoBEARS_model_object@params_table["mx01v","init"] = 0.5
M2s.diva$BioGeoBEARS_model_object@params_table["mx01v","est"] = 0.5


## INPUT the modified states list (with selected states only) into model.def.TSs
M2s.diva$lists_of_states_lists_0based[[1]] = states_list.TS1
M2s.diva$lists_of_states_lists_0based[[2]] = states_list.TS2
M2s.diva$lists_of_states_lists_0based[[3]] = states_list.TS3
M2s.diva$lists_of_states_lists_0based[[4]] = states_list.TS4

check_BioGeoBEARS_run(M2s.diva)



## run M2s.diva
res.M2s.diva <- bears_optim_run(M2s.diva)

## plot the results
setwd(wdp)
title.plot <- "DIVA on Apocynaceae M2s_stratified"
pdf("Rplot_DIVA_M2s_stratified_selRanges.pdf", width = 20, height = 100)

## States
plot_BioGeoBEARS_results(res.M2s.diva, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M2s.diva, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################



####################################################
## fit M2s-DIVA+j
#############

## define the M2s-diva model (stratified run under DIVALIKE+J model with selected range states)
M2s.diva.j <- model.def.TS

## Get the ML parameter values from the 2-parameter nested model
## (this will ensure that the 3-parameter model always does at least as good)
dstart = res.M2s.diva$outputs@params_table["d","est"]
estart = res.M2s.diva$outputs@params_table["e","est"]
jstart = 0.0001

## Input starting values for d, e
M2s.diva.j$BioGeoBEARS_model_object@params_table["d","init"] = dstart
M2s.diva.j$BioGeoBEARS_model_object@params_table["d","est"] = dstart
M2s.diva.j$BioGeoBEARS_model_object@params_table["e","init"] = estart
M2s.diva.j$BioGeoBEARS_model_object@params_table["e","est"] = estart

## Remove subset-sympatry
M2s.diva.j$BioGeoBEARS_model_object@params_table["s","type"] = "fixed"
M2s.diva.j$BioGeoBEARS_model_object@params_table["s","init"] = 0.0
M2s.diva.j$BioGeoBEARS_model_object@params_table["s","est"] = 0.0
M2s.diva.j$BioGeoBEARS_model_object@params_table["ysv","type"] = "2-j"
M2s.diva.j$BioGeoBEARS_model_object@params_table["ys","type"] = "ysv*1/2"
M2s.diva.j$BioGeoBEARS_model_object@params_table["y","type"] = "ysv*1/2"
M2s.diva.j$BioGeoBEARS_model_object@params_table["v","type"] = "ysv*1/2"

## Allow classic, widespread vicariance; all events equiprobable
M2s.diva.j$BioGeoBEARS_model_object@params_table["mx01v","type"] = "fixed"
M2s.diva.j$BioGeoBEARS_model_object@params_table["mx01v","init"] = 0.5
M2s.diva.j$BioGeoBEARS_model_object@params_table["mx01v","est"] = 0.5

## Add jump dispersal/founder-event speciation
M2s.diva.j$BioGeoBEARS_model_object@params_table["j","type"] = "free"
M2s.diva.j$BioGeoBEARS_model_object@params_table["j","init"] = jstart
M2s.diva.j$BioGeoBEARS_model_object@params_table["j","est"] = jstart

## Under DIVALIKE+J, the max of "j" should be 2, not 3 (as is default in DEC+J)
M2s.diva.j$BioGeoBEARS_model_object@params_table["j","min"] = 0.00001
M2s.diva.j$BioGeoBEARS_model_object@params_table["j","max"] = 1.99999


## INPUT the modified states list (with selected states only) into model.def.TSs
M2s.diva.j$lists_of_states_lists_0based[[1]] = states_list.TS1
M2s.diva.j$lists_of_states_lists_0based[[2]] = states_list.TS2
M2s.diva.j$lists_of_states_lists_0based[[3]] = states_list.TS3
M2s.diva.j$lists_of_states_lists_0based[[4]] = states_list.TS4

check_BioGeoBEARS_run(M2s.diva.j)



## run M2s.diva.j
res.M2s.diva.j <- bears_optim_run(M2s.diva.j)

## plot the results
setwd(wdp)
title.plot <- "DIVA+j on Apocynaceae M2s_stratified_selectedRanges"
pdf("Rplot_DIVAJ_M2s_stratified_selRanges.pdf", width = 20, height = 100)

## States
plot_BioGeoBEARS_results(res.M2s.diva.j, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
## Pie chart
plot_BioGeoBEARS_results(res.M2s.diva.j, analysis_titletxt = title.plot, plotwhat = "pie", 
                         tr = tr, tipranges = tipranges, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()
setwd(wd)
save.image("allApocynaceae_BioGeoBears_results.RData")
####################################################

## END RUN script 2: fit stratified models