## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
## Script for the biogeographic analysis used in
## "Evolution of dispersal, habit, and pollination in Africa pushed Apocynaceae diversification after the Eocene-Oligocene climate transition"
## by Bitencourt et al.; written by Nicolai Nürk, Jan 2021 (nuerk[@]uni-bayreuth.de)
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##



## Analysis: (iv) Biogeographic stochastic mapping (BSM) ## ## ## ## ## ## ## ## ## 


## (1) BSM using default DEC with selected ranges (model M1s.dec)
## (2) BSM using default DEC+J with selected ranges (model M1s.dec.j) for comparison only


## BSM in biogeobears produces several output files, which are saved in its on folder
setwd(wdBSM)
# getwd()

#######################################################
# (1.1) run BSM (model M1s.dec)
##########

## set dir
setwd(wdBSM1)

## BSM can only handle area code that consists of *single letters*
## set and run the M1-dec model (DEC with selected ranges), with *single letter* (sl) code for geographic areas
M1s.dec.bsm <- model.def.bsm
M1s.dec.bsm$states_list = states_list.sel # input the correct states list (selected ranges in Ms models) 
res.M1s.dec.bsm <- bears_optim_run(M1s.dec.bsm) # run (¡actually not needed, since we used single letter code throughout the analyses; i.e. could simply use "res.M1s.dec"…)



## Get the inputs for BSM
stochastic_mapping_inputs_list = get_inputs_for_stochastic_mapping(res = res.M1s.dec.bsm)


## do RUN 

set.seed(12345) # change for comparison of runs! e.g., do 'set.seed(as.numeric(Sys.time()))'
BSM_output = runBSM(res.M1s.dec.bsm, stochastic_mapping_inputs_list = stochastic_mapping_inputs_list, 
                    maxnum_maps_to_try = 300, nummaps_goal = n.bsm, maxtries_per_branch = 40000, save_after_every_try = TRUE, savedir = getwd(), seedval = 12345, wait_before_save = 0.01)



## get BSM output
clado_events_tables = NULL
ana_events_tables = NULL
clado_events_tables = BSM_output$RES_clado_events_tables
ana_events_tables = BSM_output$RES_ana_events_tables
#######################################################



#######################################################
# (1.2) Sumarice and save counts and maps (model M1s.dec)
##########

## Summarize stochastic map tables
## Simulate the source areas
BSMs_w_sourceAreas = simulate_source_areas_ana_clado(res.M1s.dec.bsm, clado_events_tables, ana_events_tables, areanames)
clado_events_tables = BSMs_w_sourceAreas$clado_events_tables
ana_events_tables = BSMs_w_sourceAreas$ana_events_tables
## Count all anagenetic and cladogenetic events
counts_list = count_ana_clado_events(clado_events_tables, ana_events_tables, areanames, areanames)


## print counts to files
tmpnames = names(counts_list)
cat("\n\nWriting tables* of counts to tab-delimited text files:\n(* = Tables have dimension=2 (rows and columns). Cubes (dimension 3) and lists (dimension 1) will not be printed to text files.) \n\n")
for (i in 1:length(tmpnames)) {
  cmdtxt = paste0("item = counts_list$", tmpnames[i])
  eval(parse(text=cmdtxt))
  
  # Skip cubes
  if (length(dim(item)) != 2) {
    next()
  }
  
  outfn = paste0(tmpnames[i], ".txt")
  if (length(item) == 0) {
    cat(outfn, " -- NOT written, *NO* events recorded of this type", sep="")
    cat("\n")
  } else {
    cat(outfn)
    cat("\n")
    write.table(conditional_format_table(item), file=outfn, quote=FALSE, sep="\t", col.names=TRUE, row.names=TRUE)
  } # END if (length(item) == 0)
} # END for (i in 1:length(tmpnames))


## Histogram of event counts
hist_event_counts(counts_list, pdffn = paste0("Rplot_", model_name.dec, "_histograms_of_event_counts.pdf"))


## Plot some 50 stochastic maps to PDF

cat("\n\nPlotting the first few stochastic maps to file: \n'Rplot_BSM_DEC_M1s.pdf'\n\n")

## Loop through the maps and plot to PDF
pdf(file = "Rplot_BSM_M1s.dec.pdf", width = 20, height = 100)
for (i in 1:n.p.bsm){
  clado_events_table = clado_events_tables[[i]]
  analysis_titletxt = paste0(model_name.dec, " - Stochastic Map #", i, "/", n.p.bsm)
  plot_BSM(results_object = res.M1s.dec.bsm, clado_events_table = clado_events_table, stratified = stratified, 
           analysis_titletxt = analysis_titletxt, addl_params = list("j"), label.offset = 0.5, plotwhat = "text", cornercoords_loc = scriptdir, 
           colors_list_for_states = colors_list_for_states, include_null_range = TRUE, root.edge = TRUE, 
           show.tip.label = FALSE, plotsplits = FALSE)
  }

dev.off()
#######################################################



#######################################################
# (1.3) Check that ML ancestral state/range probabilities and
# the mean of the BSMs approximately line up (model M1s.dec)
##########

check_ML_vs_BSM(res.M1s.dec.bsm, clado_events_tables, paste0("Rplot_", model_name.dec), tr = NULL, plot_each_node = FALSE, linreg_plot = TRUE, MultinomialCI = TRUE)

#######################################################
## set wd back to BSM
setwd(wdBSM)




#######################################################
# (2.1) run BSM (model M1s.dec.j)
##########

## Make folder for THIS BSM run and copy the distribution file with area names in single letters (required for BSM)
setwd(wdBSM2)

## BSM can handle only an area code that consists of *single letters* (!)
## so we simply repeat the run for the best model (s. results from 'script_0_master….R') here:
## set and run the M1-dec model (DEC with selected ranges), with *single letter* (sl) code for geographic areas
M1s.dec.j.bsm <- model.def.bsm
dstart = res.M1s.dec.bsm$outputs@params_table["d","est"]
estart = res.M1s.dec.bsm$outputs@params_table["e","est"]
jstart = 0.0001
M1s.dec.j.bsm$BioGeoBEARS_model_object@params_table["d","init"] = dstart
M1s.dec.j.bsm$BioGeoBEARS_model_object@params_table["d","est"] = dstart
M1s.dec.j.bsm$BioGeoBEARS_model_object@params_table["e","init"] = estart
M1s.dec.j.bsm$BioGeoBEARS_model_object@params_table["e","est"] = estart
M1s.dec.j.bsm$BioGeoBEARS_model_object@params_table["j","type"] = "free"
M1s.dec.j.bsm$BioGeoBEARS_model_object@params_table["j","init"] = jstart
M1s.dec.j.bsm$BioGeoBEARS_model_object@params_table["j","est"] = jstart
M1s.dec.j.bsm$states_list = states_list.sel # input the correct states list (selected ranges in Ms models) 
check_BioGeoBEARS_run(M1s.dec.j.bsm)

## run res.M1s.dec.j.bsm
res.M1s.dec.j.bsm <- bears_optim_run(M1s.dec.j.bsm)

## plot the results
title.plot <- "DEC+j on Apocynaceae M1s_unconstrained_selectedRanges_forBSM"
pdf("Rplot_DECJ_M1s_unconstrained_selRanges_BSM.pdf", width = 20, height = 100)
## States
plot_BioGeoBEARS_results(res.M1s.dec.j.bsm, analysis_titletxt = title.plot, plotwhat = "text", 
                         tr = tr, tipranges = tipranges.bsm, cornercoords_loc = scriptdir, addl_params=list("j"), include_null_range = TRUE, 
                         label.offset = 0.45, tipcex = 0.4, statecex = 0.5, splitcex = 0.4, titlecex = 1.2, 
                         plotsplits = FALSE)
dev.off()



## Get the inputs for BSM
stochastic_mapping_inputs_list.j = get_inputs_for_stochastic_mapping(res = res.M1s.dec.j.bsm)


## do RUN 
set.seed(12345) # change among runs to compare
BSM_output.j = runBSM(res.M1s.dec.j.bsm, stochastic_mapping_inputs_list = stochastic_mapping_inputs_list.j, 
                    maxnum_maps_to_try = 300, nummaps_goal = n.bsm, maxtries_per_branch = 40000, save_after_every_try = TRUE, savedir = getwd(), seedval = 12345, wait_before_save = 0.01)



## get BSM output
clado_events_tables.j = NULL
ana_events_tables.j = NULL
clado_events_tables.j = BSM_output.j$RES_clado_events_tables
ana_events_tables.j = BSM_output.j$RES_ana_events_tables
#######################################################



#######################################################
# (2.2) Sumarice and save counts and maps (model M1s.dec.j)
##########

## Summarize stochastic map tables
## Simulate the source areas
BSMs_w_sourceAreas = simulate_source_areas_ana_clado(res.M1s.dec.j.bsm, clado_events_tables.j, ana_events_tables.j, areanames)
clado_events_tables.j = BSMs_w_sourceAreas$clado_events_tables
ana_events_tables.j = BSMs_w_sourceAreas$ana_events_tables
## Count all anagenetic and cladogenetic events
counts_list = count_ana_clado_events(clado_events_tables.j, ana_events_tables.j, areanames, areanames)


## print counts to files
tmpnames = names(counts_list)
cat("\n\nWriting tables* of counts to tab-delimited text files:\n(* = Tables have dimension=2 (rows and columns). Cubes (dimension 3) and lists (dimension 1) will not be printed to text files.) \n\n")
for (i in 1:length(tmpnames)) {
  cmdtxt = paste0("item = counts_list$", tmpnames[i])
  eval(parse(text=cmdtxt))
  
  # Skip cubes
  if (length(dim(item)) != 2) {
    next()
  }
  
  outfn = paste0(tmpnames[i], ".txt")
  if (length(item) == 0) {
    cat(outfn, " -- NOT written, *NO* events recorded of this type", sep="")
    cat("\n")
  } else {
    cat(outfn)
    cat("\n")
    write.table(conditional_format_table(item), file=outfn, quote=FALSE, sep="\t", col.names=TRUE, row.names=TRUE)
  } # END if (length(item) == 0)
} # END for (i in 1:length(tmpnames))


## Histogram of event counts
hist_event_counts(counts_list, pdffn = paste0("Rplot_", model_name.dec.j, "_histograms_of_event_counts.pdf"))


## Plot some 50 stochastic maps to PDF

cat("\n\nPlotting the first few stochastic maps to file: \n'Rplot_BSM_DEC_M1s.pdf'\n\n")

## Loop through the maps and plot to PDF
pdf(file = "Rplot_BSM_M1s.dec.j.pdf", width = 20, height = 100)
for (i in 1:n.p.bsm){
  clado_events_table = clado_events_tables.j[[i]]
  analysis_titletxt = paste0(model_name.dec.j, " - Stochastic Map #", i, "/", n.p.bsm)
  plot_BSM(results_object = res.M1s.dec.j.bsm, clado_events_table = clado_events_table, stratified = stratified, 
           analysis_titletxt = analysis_titletxt, addl_params = list("j"), label.offset = 0.5, plotwhat = "text", cornercoords_loc = scriptdir, 
           colors_list_for_states = colors_list_for_states, include_null_range = TRUE, root.edge = TRUE, 
           show.tip.label = FALSE, plotsplits = FALSE)
}

dev.off()
#######################################################



#######################################################
# (2.3) Check that ML ancestral state/range probabilities and
# the mean of the BSMs approximately line up (model M1s.dec.j)
##########

check_ML_vs_BSM(res.M1s.dec.j.bsm, clado_events_tables.j, paste0("Rplot_", model_name.dec.j), tr = NULL, plot_each_node = FALSE, linreg_plot = TRUE, MultinomialCI = TRUE)


#######################################################

## set wd back to wd.master
setwd(wd)

## END run BSM 



