## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
## script for BAMM diversification rate analysis used in
## "Evolution of dispersal, habit, and pollination in Africa pushed Apocynaceae diversification after the Eocene-Oligocene climate transition"
## by Bitencourt et al.; written by Nicolai Nürk, Mar 2021 (nuerk[@]uni-bayreuth.de)
##
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##

library(BAMMtools)

#for later:
load("BAMM_results_Apocynaceae_1.RData")

##########################################
#0. estimate appropriate priors
##########################################
tree <- read.nexus("ApocynTree_Fishbein2018_n1041_newNames20210616.tre")

#before analysis: estimate suitable priors
setBAMMpriors(tree, total.taxa = 5350)

## adjust script to the expectedNumberOfShifts used (i.e. "1")
PriorShifts <- 1
## using search and replace:
# " Shift Prior 1.0 "
# " BAMM_1 "

##########################################




##########################################
# 1. load data
##########################################

tree <- read.tree("ApocynTree_Fishbein2018_n1041_newNames20210616.tre")

# event data & mcmc
edata <- getEventData(tree, eventdata = "BAMM_1_Apocyn_event_data.txt", burnin=0.2)
mcmcout <- read.csv("BAMM_1_Apocyn_mcmc_out.txt", header=T)
##########################################



##########################################
# 2. accessing MCMC convergence
##########################################
par(mfrow = c(1, 1))
pdf(file = "Rplot_Apocyn_BAMM_1_mcmc.pdf")
plot(mcmcout$logLik ~ mcmcout$generation)
title(main = "Shift Prior 1.0")
dev.off()

# discard 10% as burnin
burnstart <- floor(0.2 * nrow(mcmcout))
postburn <- mcmcout[burnstart:nrow(mcmcout), ]
pdf(file = "Rplot_Apocyn_BAMM_1_mcmc_postburn.pdf")
plot(postburn$logLik ~ postburn$generation)
title(main = "Shift Prior 1.0")
dev.off()


#check ESS for the post-burnin samples; 
#we want these to be at least 200 
#(and 200 is on the low side, but might be reasonable for very large datasets)
library(coda)
effectiveSize(postburn$N_shifts)
effectiveSize(postburn$logLik)
##########################################


##########################################
#3. compare prior and posterior & BFs
##########################################

#plot the prior and the posterior distributions
pdf(file = "Rplot_Apocyn_BAMM_1_priorposterior_hist.pdf")
plotPrior(mcmcout, expectedNumberOfShifts = PriorShifts, burnin = 0.1, priorCol = "grey90", postCol = "royalblue", density = 15)
title(main = "Shift Prior 1.0")
dev.off()

#compute the posterior probabilities of models sampled using BAMM
post_probs <- table(postburn$N_shifts) / nrow(postburn)
names(post_probs)
#post_probs is now a vector of model posterior probabilities. 
#We can look at these, we can plot them, we can compute posterior odds ratios, and so on. 
post_probs

plot(post_probs)


#compute Bayes factors
## read like: row if favored over column below the diagonal
sink("BAMM_results_Apocynaceae_1_BFs.txt", split = TRUE, append = FALSE)
( bfmat <- computeBayesFactors(mcmcout, expectedNumberOfShifts = PriorShifts, burnin=0.1) )
sink()


## Cohort matrix
cmat <- getCohortMatrix(edata)
pdf(file = "Rplot_Apocyn_BAMM_1_cohort.pdf")
cohorts(cmat, edata)
dev.off()


## Marginal odds ratio
tree.mor <- marginalOddsRatioBranches(edata, expectedNumberOfShifts = PriorShifts)
pdf("Rplot_Apocyn_BAMM_1_MOR_node&edgeNumbers.pdf", width = 20, height = 100)
plot.phylo(tree.mor, cex = 0.4)
edgelabels(, cex = .5)
nodelabels(, cex = .4)
dev.off()

##########################################



##########################################
# 4. plot regime trees
##########################################
#plot the mean marginal posterior density rates with a histogram of rates distribution throughout the tree as legend
pdf(file = "Rplot_Apocyn_BAMM_1_phyloratePlots.pdf")
par(mfrow = c(1,2))
ratesHistogram(plot.bammdata(edata, spex = "s", method = "phylogram", rbf = 0.01, logcolor = TRUE), plotBrks = FALSE, xlab = "speciation rate"); title(main = "Shift Prior 1.0")
ratesHistogram(plot.bammdata(edata, spex = "netdiv", method = "phylogram", rbf = 0.01, logcolor = TRUE), plotBrks = FALSE, xlab = "net diversification rate")
ratesHistogram(plot.bammdata(edata, spex = "e", method = "phylogram", rbf = 0.01, logcolor = TRUE), plotBrks = FALSE, xlab = "extinction rate")
dev.off()

#Bayesian credible sets of shift configurations
css <- credibleShiftSet(edata, expectedNumberOfShifts = PriorShifts, threshold=5, set.limit = 0.15)
summary(css)
#number of distinct shift configurations
css$number.distinct
##plot n save
pdf(file = "Rplot_Apocyn_BAMM_1_css.pdf")
plot.credibleshiftset(css, 6, rbf = 0.01, logcolor = TRUE)
title(sub = "Shift Prior 1.0", line = -1)
dev.off()

#the single best from css
pdf(file = "Rplot_Apocyn_BAMM_1_css_best.pdf")
plot.credibleshiftset(css, 1, rbf = 0.01, logcolor = TRUE, legend = FALSE, method = "polar", use.plot.bammdata = FALSE) # "use.plot.bammdata = TRUE" for phylorate tree
title(sub = "Shift Prior 1.0", line = -1)
dev.off()

#the single best from css with legend
pdf(file = "Rplot_Apocyn_BAMM_1_css_best_leg.pdf")
plot.credibleshiftset(css, 1, rbf = 0.01, logcolor = TRUE, legend = TRUE, method = "polar", use.plot.bammdata = FALSE)
# title(sub = "Shift Prior 1.0", line = -1)
dev.off()

# and the best
best <- getBestShiftConfiguration(edata, expectedNumberOfShifts = PriorShifts, threshold = 5)

# plot
pdf(file = "Rplot_Apocyn_BAMM_1_best.pdf")
par(mfrow = c(1,1))
plot.bammdata(best, spex = "s", lwd = 2, legend = F, logcolor = TRUE) #breaksmethod='linear', color.interval=c(NA, 1)
addBAMMshifts(best, cex=1.5, pch = 8, col = "black") #stars
addBAMMshifts(best, cex=1.5, pch = 21, col = "black", bg = "grey80")
title('best shift configuration on speciation rates; Shift Prior 1.0', cex.sub = 2.2, line= -1)
dev.off()
##########################################


######################
#Plotting rate-through-time curve
##########################################
pdf(file = "Rplot_Apocyn_BAMM_1_rtt.pdf")
par(mfrow = c(1, 3))
plotRateThroughTime(edata, ratetype="netdiv", xlim=c(80, 0), ylim=c(0.01,0.4)) # speciation
#text(x=30, y=0.7, label="Apocynaceae netdiv", cex=1.2, font=4);
plotRateThroughTime(edata, ratetype="speciation", xlim=c(80, 0), ylim=c(0.01,0.5)) # 
plotRateThroughTime(edata, ratetype="extinction", xlim=c(80, 0), ylim=c(0.01,0.25))
dev.off()
##########################################




###############################
## Clade-specific evolutionary rates
##########################################

## The mean rate  for Apocynaceae
allrates <- getCladeRates(edata)
# 'allrates' is a list with speciation and extinction components, with the mean rate across all species for each sample in the posterior. 
par(mfrow = c(1,2))
hist(allrates$lambda-allrates$mu, 100, col = "grey85", border = 0, main = "Mean Apocynaceae netDiv rates")
abline(v = mean(allrates$lambda-allrates$mu), col = "red")
abline(v = quantile(allrates$lambda-allrates$mu, c(0.05, 0.95)), col = "green")

## the background rate (all Apocynaceae excluding clades with a rate shift) 
## First, get the shift nodes from MAP
shiftnodes.best <- sort(getShiftNodesFromIndex(best, 1))
# shiftnodes.best.css <- css$shiftnodes[1]
# shiftnodes.best <- list(sort(getShiftNodesFromIndex(best, 1)))
# identical(shiftnodes.best.css, shiftnodes.best)
back.rates <- getCladeRates(edata, node = c(1042, shiftnodes.best), nodetype = c("include", "exclude", "exclude", "exclude", "exclude", "exclude", "exclude", "exclude", "exclude", "exclude", "exclude", "exclude", "exclude"))


## Compute the mean background speciation rate for Apocynaceae and estimate the 95% highest posterior density (HPD):
## and assamle into a matrix

## set up results matrix
div.rates <- matrix(nrow = 1+ length(shiftnodes.best), ncol = 3, byrow = TRUE)
colnames(div.rates) <- c("meanNetDiv", "5%HPD", "95%HPD")
rownames(div.rates) <- c("Apocynaceae.root", as.character(shiftnodes.best))
## add the background rate
div.rates[1,1] <- mean(back.rates$lambda-back.rates$mu)
div.rates[1,2] <- quantile(back.rates$lambda-back.rates$mu, probs = 0.05)
div.rates[1,3] <- quantile(back.rates$lambda-back.rates$mu, probs = 0.95)

## loop over all shift nodes
for(i in 1:length(shiftnodes.best)) {
	cl <- getCladeRates(edata, node = shiftnodes.best[i])
	clm <- mean(cl$lambda-cl$mu)
	cl5 <- as.numeric(quantile(cl$lambda-cl$mu, probs = 0.05))
	cl95 <- as.numeric(quantile(cl$lambda-cl$mu, probs = 0.95))
	div.rates[i+1, ] <- c(clm, cl5, cl95)
	}


## add background rates to the histogram
hist(back.rates$lambda-back.rates$mu, 100, col = "grey60", border = 0, add = FALSE, main = "Background netDiv rates")
abline(v = div.rates[1,1], col = "darkred")
abline(v = quantile(back.rates$lambda-back.rates$mu, c(0.05, 0.95)), col = "darkgreen")
##########################################




######################
# prep main RESULTS (used in the paper)
##########################################

## TABLE 1
## model-averaged shifts from MAP configuration
res.table <- best$eventData[[1]]
# shiftnodes.best <- getShiftNodesFromIndex(best, 1)
# isShift <- best$eventData[[1]]$node %in% shiftnodes.best
# time <- best$eventData[[1]]$time[isShift]

## sort table according to node numbers
( res.table <- res.table[order(as.numeric(res.table[, "node"])), ] )

## extract the marginal probabilities from the MAP (add the '0' for the root node)
res.table$margProbs <- c(0, css$marg.probs[as.character(css$shiftnodes[[1]])])

## add speciation rates
( res.table <- round(cbind(res.table, div.rates), 3) )

## save 
write.table(res.table, file = "BAMM_results_Apocynaceae_1_MAP_shifts.txt", quote = FALSE, sep = "\t")




## FIGURE 3:
## Get the best shift (MAP) configuration with shifts scaled to marginal probabilities
## Note that we use ape::plot.phylo , so all shifts are placed at node positions, not at the branch segemnt, where the shift is estimated (cf. "Rplot_Apocyn_BAMM_1_css_best_leg.pdf" and Table 1)

## get the relevant rate shift nodes from the best shift configuration
# nodes.best <- css$shiftnodes[[1]]
shiftnodes.best <- sort(getShiftNodesFromIndex(best, 1))

## get scaling and colors for shift nodes based on their importance
library(gplots);
pal <- rich.colors(20);
cexmin <- 1;
cexmax <- 6;

# Get marginal probabilities associated with each node in the the 
#		maximum shift credibility configuration
probvec.best <- css$marg.probs[as.character(shiftnodes.best)]

size.best <- cexmin + probvec.best*(cexmax - cexmin);


## Get colors for nodes based on marginal shift probs
colvec.best <- rep(pal[length(pal)], length(shiftnodes.best));

cuts <- seq(0, 1, length.out=length(pal)+1); 

for (i in 1:(length(cuts) - 1)){
  for (k in 1:length(probvec.best)){
    if (probvec.best[k] > cuts[i] & probvec.best[k] <= cuts[i+1]){
      colvec.best[k] <- pal[i];
    }
  }	
}

lmat <- matrix(c(1,1,1,1,1,1,1, 2), nrow=1);

edgewid <- rep(0.5, length(tree$edge.length));
edgecol <- rep('gray60', length(tree$edge.length));


quartz.options(height=5, width=17);

## plot

layout(lmat)
## First, the MAP shift tree:
pdf(file = "Rplot_Apocyn_BAMM_1_MAP_marginalP.pdf")
plot.phylo(tree, show.tip.label = FALSE, type = "fan")
axisPhylo()
nodelabels(node = shiftnodes.best, cex = size.best, pch = 21, bg = colvec.best) #bg = colvec : for colored shifts
title(main = "Shift Prior 1.0 & MAP shift configuration")

## now the scale:
plot.new();
par(mar=c(3,0,3,6));
plot.window(xlim=c(0,4), ylim=c(-0.3, 1.3));
for (i in 1:(length(cuts)-1)){
  xco <- c(0,2,2, 0);
  yco <- c(cuts[i], cuts[i], cuts[i+1], cuts[i+1]);
  polygon(x=xco, y=yco, col=pal[i]);
}
axis(side=4,at=seq(0,1, by=0.2), cex.axis=2, font=2, las=1, pos=2.2)
mtext(text='Shift probability', side=4, line= 3.1, cex=1.5) 

dev.off()




## plot node numbers with MAP shifts config

title.plot <- "BAMM best shifts on Apocynaceae  <-  node numbers"
pdf("Rplot_Apocyn_BAMM_1_best_nodeNumbers.pdf", width = 20, height = 100)
plot.phylo(tree, cex = 0.4)
axisPhylo()
nodelabels(node = css$shiftnodes[[1]], frame = "circle", bg = colvec.best)
dev.off()

##########################################
save.image(file = "BAMM_results_Apocynaceae_1.RData")

## END














## COMPARE (not run)
#################################
# Max shift credibility configuration on the phylogeny

# Compute the marginal shift probabilities on branches
marg_tree <- marginalShiftProbsTree(edata);

# Compute the cumulative shift probabilities for branches:
cst <- cumulativeShiftProbsTree(edata);

# Compute the maximum shift credibility configuration:
# This is the joint distribution of shift events that maximizes the 
#	marginal probability of the data.
msc <- maximumShiftCredibility(edata);


# get the relevant rate shift nodes from the 
#	maximum shift credibility configuration
nodes <- getShiftNodesFromIndex(edata, index = msc$sampleindex);

# sort(nodes.msc)


# get scaling and colors for shift nodes based on
#			their importance

library(gplots);
pal <- rich.colors(20);


cexmin <- 1;
cexmax <- 6;



# Get marginal probabilities associated with each node in the the 
#		maximum shift credibility configuration
probvec <- numeric(length(nodes));

for (i in 1:length(nodes)){
  probvec[i] <- marg_tree$edge.length[marg_tree$edge[,2] == nodes[i] ];
}
size <- cexmin + probvec*(cexmax - cexmin);


## Get colors for nodes based on marginal shift probs
colvec <- rep(pal[length(pal)], length(nodes));

cuts <- seq(0, 1, length.out=length(pal)+1); 

for (i in 1:(length(cuts) - 1)){
  for (k in 1:length(probvec)){
    if (probvec[k] > cuts[i] & probvec[k] <= cuts[i+1]){
      colvec[k] <- pal[i];
    }
  }	
}


lmat <- matrix(c(1,1,1,1,1,1,1, 2), nrow=1);


edgewid <- rep(0.5, length(tree$edge.length));
edgecol <- rep('gray60', length(tree$edge.length));

#edgewid[cst$edge.length > 0.9] <- 0.7;
#edgecol[cst$edge.length > 0.9] <- pal[10];


## First, the marginal shift probability tree:
quartz.options(height=5, width=17);



## plot n save
# pdf(file = "Rplot_Apocyn_BAMM_1_msc_marginalP.pdf")
layout(lmat);
#plot.phylo(as.phylo.bammdata(edata), show.tip.label=F, edge.width=0.8, edge.color='gray40', no.margin=T, type = 'p', direction='upwards');
#plot.bammdata(best, spex = "s", lwd = 2, legend = F, logcolor = TRUE) #breaksmethod='linear', color.interval=c(NA, 1)
plot.phylo(tree, show.tip.label = FALSE)
axisPhylo()
nodelabels(node = nodes, cex = size, pch = 21, bg = colvec) #bg = colvec #for colored shifts
title(main = "Shift Prior 1.0")

plot.new();
par(mar=c(3,0,3,6));
plot.window(xlim=c(0,4), ylim=c(-0.3, 1.3));
for (i in 1:(length(cuts)-1)){
  xco <- c(0,2,2, 0);
  yco <- c(cuts[i], cuts[i], cuts[i+1], cuts[i+1]);
  polygon(x=xco, y=yco, col=pal[i]);
}
axis(side=4,at=seq(0,1, by=0.2), cex.axis=2, font=2, las=1, pos=2.2)
mtext(text='Shift probability', side=4, line= 3.1, cex=1.8) 



##########################################
##Compare MSC (best: maximum shift credibility configuration) and MAP (best: highest maximum a posteriori probability)
## cf. help file: 
## This [MSC] is one point estimate of the overall "best" rate shift configuration. 
## Following an MCMC simulation, the marginal shift probabilities on each individual branch are computed using marginalShiftProbsTree. 
## The shift configuration that maximizes the product (or sum, if specified) of these marginal branch-specific shift probabilities is the maximum shift credibility configuration.
## This option is only recommended if you have no clear "winner" in your credible set of shift configurations (see credibleShiftSet). 
## If you have a number of largely-equiprobable shift configurations in your 95% credible set, you may wish to try this function as an alternative for identifying a single best shift configuration. 
## Otherwise, it is recommended that you present the shift configuration with the maximum a posteriori probability (see getBestShiftConfiguration).


#Plot and compare 
#title.plot <- "MSC & MAP shift configuration on MSC speciation rates; Shift Prior 1.0"
pdf("Rplot_Apocyn_BAMM_1_map_vs_msc_shifts.pdf", width = 20, height = 20)
plot.bammdata(edata, spex = "s", lwd = 2, legend = TRUE, logcolor = TRUE, method = "polar") 
addBAMMshifts(edata, method='polar', index = msc$sampleinde, col = "red", bg = "red", cex = 3) # MSC
addBAMMshifts(best, method='polar', col = "green", bg = NULL, pch = 22, cex = 3) # MAP
legend("topleft", legend = c("msc", "map"), pch = c(21, 22), col = c("red", "green"), pt.bg = c("red", "green"), bty = "n")
title('MSC & MAP (best) shift configuration on MSC speciation rates; Shift Prior 1.0', cex.sub = 2.2, line= 1)
dev.off()


## plot node numbers with MSC shifts config
title.plot <- "BAMM msc shifts on Apocynaceae  <-  node numbers"
pdf("Rplot_Apocyn_BAMM_1_msc_nodeNumbers.pdf", width = 20, height = 100)
plot.phylo(tree, cex = 0.4)
axisPhylo()
nodelabels(node = nodes, frame = "circle", bg = colvec)
dev.off()

## plot node numbers with MAP shifts config
title.plot <- "BAMM best shifts on Apocynaceae  <-  node numbers"
pdf("Rplot_Apocyn_BAMM_1_best_nodeNumbers.pdf", width = 20, height = 100)
plot.phylo(tree, cex = 0.4)
axisPhylo()
nodelabels(node = css$shiftnodes[[1]], frame = "circle", bg = colvec)
dev.off()

