## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
## script for MiSSE diversification rate analysis used in
## "Evolution of dispersal, habit, and pollination in Africa pushed Apocynaceae diversification after the Eocene-Oligocene climate transition"
## by Bitencourt et al.; written by Nicolai Nürk, Nov 2020 (nuerk[@]uni-bayreuth.de)
##
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
##
## The script produces
## (1) parameter estimation & 
## (2) model averaging using MiSEE (Missing State Speciation and Extinction) 
## in hisse v1.9.10
##
## following https://cran.r-project.org/web/packages/hisse/vignettes/MiSSE-vignette.pdf
##
## for instructions on installation of required packages
## see: https://github.com/thej022214/hisse
## DON'T use:
# remotes::install_github("thej022214/hisse") [at Nov 2020: this version seems to have the help files broken]
## USE: 
# install.packages("hisse") # = version hisse_1.9.10
## 
##
##
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
##
## The script runs the following analyses 
## assuming Apocynaceae phylogeny sampled 20% of taxonomic diversity
##  
##  (1) single MiSEE (Missing State Speciation and Extinction: Examines shifts in diversification in relation to hidden states ONLY): training
##  (2) MiSSEGreedy: fit several rate (hidden state) models
##
##
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##
## 
## Files needed:
##
##  SCRIPTS
##  (1) this script:  "script_MiSEE_models_20210204clean.R"
##  (2) "myFUN_plot.misse.states.R" - modified plotting function, 
##
##  DATA
##  (3) dated phylogeny: "ApocynTree_Fishbein2018_n1041_Llad_newNames.tre"
##
##
## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ## ##


rm(list = ls())
library(hisse)
source("myFUN_plot.misse.states.R")




## Load the RESULTS
load("Apocyn_MiSSEE_allResults.RData")

## or — START AGAIN analyses:

sink(file = "runtime.txt", append = FALSE)
cat("\n\nStart run: 'script_MiSEE_models_20210204clean.R' \non iMac 2017 (OS 10.13.6, 4.2 GHz i7, 64 GB memory):\n\n")
date()
sink()
########################################
## load data 
####################
phy <- read.tree("ApocynTree_Fishbein2018_n1041_newNames20210616.tre")

# ## possible:
# ## load the single results objects from "MiSSEGreedy" and "MarginReconMiSSE"
# ## which are available as RDS files (since these take time to finish)

# res103 <- readRDS("results_MiSSEGreedy_103mods_Apocyn.RDS")
# res103.recons <- readRDS("./results_MiSSEGreedy_103mods_Apocyn_marginalRecons.RDS")
# res.two.rate <- readRDS("results_MiSSE_two.rates_Apocyn.RDS")
# res.two.rate.recon <- readRDS("results_MiSSE_two.rates.recons_Apocyn.RDS")
########################################


########################################
## DON'T RUN: fit single MiSSE models (just to see what it does)
####################
#
# ## what's available on CPUs 
# parallel::detectCores()
#
# ## fit a single rate model
# turnover <- c(1)
# eps <- c(1)
# one.rate <- MiSSE(phy, f = .20, turnover = turnover, eps = eps, sann = TRUE, sann.its = 1000, dt.threads = 8)
# ##  marginal reconstruction
# ONE.rate.recon <- MarginReconMiSSE(phy = phy, f = 0.2,  hidden.states = 1, pars = two.rate$solution, aic = two.rate$AIC, n.cores = 8)

# ## fit a two rate model
# turnover <- c(1,2)
# eps <- c(1,1)
# res.two.rate <- MiSSE(phy, f = 0.2, turnover = turnover, eps = eps, sann = FALSE, sann.its = 1000, dt.threads = 8)
# ##  marginal reconstruction
# res.two.rate.recon <- MarginReconMiSSE(phy = phy, f = 0.2,  hidden.states = 2, pars = two.rate$solution, aic = two.rate$AIC, n.cores = 8)
# ## etc. …
########################################



########################################
## run MiSSEGreedy: fit several rate (hidden state) models using AICc as stop criterion
####################

## (1) Set up some MiSSE models, varying either the number of parameters for turnover or extinction fraction
## = 103 models
combs <- generateMiSSEGreedyCombinations(vary.both = FALSE)

## (2) or keep eps fixed at 0.9 and 0.0 (eps = extinction fraction = extinction rate divided by speciation rate = µ/lamda)
## = 52 models 
combs0.09 <- generateMiSSEGreedyCombinations(fixed.eps.tries=c(0, 0.9))


## RUN the optimization over all (103) MiSSE models
## one should test the effect of different starting.vals on maximum likelihood optimization…
## this can take some time
res103 <- MiSSEGreedy(phy, f = .20, possible.combos = combs, n.cores = parallel::detectCores(), save.file = "missegreedy.results103_1.RData")  

## save these MiSSE results
saveRDS(res103, file = "results_MiSSEGreedy_103mods_Apocyn.RDS")



## RUN the rate class reconstruction by looping over all 103 models evaluated using the marginal reconstruction algorithm:
## The marginal probability of state combination i for a focal node is the overall likelihood of the tree and data 
## when the state of the focal node is fixed in state combination i.
res103.recons <- as.list(1:length(res103)) 
## this takes time!
for (model_index in 1:length(res103)){
        nturnover <- length(unique(res103[[model_index]]$turnover))
        # eps <- res103[[model_index]]$fixed.eps
        misse_recon <- MarginReconMiSSE(phy = res103[[model_index]]$phy, f = .2,
                                        hidden.states = nturnover,
                                        # fixed.eps = eps, # this is not in the example code in the misse vignette! (missing?)
                                        pars = res103[[model_index]]$solution, aic = res103[[model_index]]$AIC,
                                        n.cores = parallel::detectCores())
        res103.recons[[model_index]] <- misse_recon
        }

## save these MiSSE reconstructions
saveRDS(res103.recons, file = "results_MiSSEGreedy_103mods_Apocyn_marginalRecons.RDS")



save.image("Apocyn_MiSSEE_allResults.RData")
########################################



########################################
## Analyze the MiSSEGreedy results
####################

## (1) get results for res103 in a table
##
## extract relevant parameter estimates from the res103
## log likelihoods
loglik <- unlist(lapply(res103, "[[", "loglik"))
## Akaike information criterion (AIC)
AIC <- unlist(lapply(res103, "[[", "AIC"))
##  sample size corrected AIC (AICc)
AICc <- unlist(lapply(res103, "[[", "AICc"))
## calc deltaAIC
AICc.detla <- AICc-min(AICc)
## calculate AIC.weights
# AIC.weights <- GetAICWeights(res103, criterion = "aicc") # returns an error
## manually so
AICcw <- exp( -0.5 * AICc.detla) / sum( exp( -0.5 * AICc.detla) )

## combine res103 into a table
combs.resAll <- combs
combs.resAll$loglik <- loglik
combs.resAll$AICc <- AICc
combs.resAll$deltaAICc <- AICc.detla
combs.resAll$AICc.wi <- AICcw

## save the 103 results table
( combs.resAll <- combs.resAll[order(as.numeric(combs.resAll[, "deltaAICc"])), ] )
write.table(combs.resAll, file = "results_MiSSE_103modelfit_Apocynaceae.txt", quote = FALSE, sep = "\t")



## (2) get results for the 52 models with fixed eps = c(0.0, 0.9): generates "res0.09.recons"
##
## FIRST, select the 'fixed eps' models (from combs.resAll)
combs0.09.res <- combs.resAll[combs.resAll$eps %in% 0, c("turnover", "eps", "fixed.eps", "loglik", "AICc")]

## calc ∆AICc among those 52 models
combs0.09.res$deltaAICc <- combs0.09.res$AICc - min(combs0.09.res$AICc)
## calc AICc.wi among those 52 models
combs0.09.res$AICc.wi <- exp( -0.5 * combs0.09.res$deltaAICc) / sum( exp( -0.5 * combs0.09.res$deltaAICc) )

## get the row number of the 52 models with fixed eps
combs0.09.res$no52 <- rownames(combs0.09)

## sort acc to ∆AICc
( combs0.09.res <- combs0.09.res[order(as.numeric(combs0.09.res[, "deltaAICc"])), ] )

## save the 52 results table
write.table(combs0.09.res, file = "results_MiSSE_52modelfit_Apocynaceae.txt", quote = FALSE, sep = "\t")





## SECOND, extract the 52 models from the res103.recons
res0.09.recons <- res103.recons[as.numeric(row.names(combs0.09.res))]

## get model averaged rates
res0.09avg.rates <- GetModelAveRates(res0.09.recons, type = "both")


## calculate confidence intervals of estimates 
## for the most likely model (No 52) in "res103"
res.bestMod.ci <- SupportRegionMiSSE(res103[[as.numeric(rownames(combs.resAll[1])[1])]])

## save CIs best model
write.table(res.bestMod.ci$ci, file = "results_MiSSE_bestmodel52outof103_CI_Apocynaceae.txt", quote = FALSE, sep = "\t")
## how to do this for all models in the list ? (for loop probably…)
## how to model average CIs ? (makes few sense maybe…)
########################################

  
  
########################################
## plot MiSSEGreedy
####################


## specify what to plot
param = "net.div"   # possible: rate.param = “turnover”, “net.div”, “speciation”, “extinction”, “extinction.fraction”
leg = "all"         # possible: legend = "none", "traditional", "tips", "internal", “all”



# ## plot rates for the two.rate model
# plot.misse.states(res.two.rate.recon, rate.param = param, show.tip.label = FALSE, type = "fan", fsize = .25, legend = leg); title(main = param, line = -20)
# ## compare to extinction rates (two.rate model)
# plot.misse.states(res.two.rate.recon, rate.param = "extinction", show.tip.label = FALSE, type = "fan", fsize = .25, legend = leg); title(main = paste0("extinction rates"), line = -20)



## plot the 52 models with eps fixed to 0, 0.9 
## averaged rates & maxval (!)
pdf("Rplot_MiSSE_Apocyn_52M.averaged.pdf", width = 10, height = 12)
my.plot.misse.states(res0.09.recons, rate.param = param, show.tip.label = FALSE, type = "fan", fsize = .25, legend = leg, 
                     rate.colors = c("darkblue", "lightblue", "red", "darkred"), 
                     maxval = 1.35) 
title(main = paste0(param, "\n\naverage of 52 models \n(with eps fixed at 0.0 and 0.9)"), line = -1) # ', line = -30' can be adjusted to plot size
dev.off()












# ## *select* appropriate maxval
# ## for skewness of rates — data = log(rates)!
# rates.tips <-  log(my.ConvertManyToRate(res0.09.recons, rate.param = param, "tip.mat"))
# hist(rates.tips)
# max(rates.tips)
# min(rates.tips)
# rates.tips[rates.tips > 0.3] <- 0.3 #here 0.3 is set as max rate value (see plotting above: maxval = 0.3)
# max(rates.tips)
# min(rates.tips)
# hist(rates.tips)
# hist(exp(rates.tips))
# 
# rates.internal <- log(my.ConvertManyToRate(res0.09.recons, rate.param= param, "node.mat"))
# hist(rates.internal)
# max(rates.internal)
# min(rates.internal)
# rates.internal[rates.internal > 0.3] <- 0.3 #here 0.3 is set as max rate value (see plotting above: maxval = 0.3)
# max(rates.tips)
# max(rates.internal)
# min(rates.internal)
# hist(rates.internal)
# hist(exp(rates.internal))

save.image("Apocyn_MiSSEE_allResults.RData")
########################################

sink(file = "runtime.txt", append = TRUE, split = TRUE)
cat("\nEnd run:\n")
date()
cat("\n\nusing\n\n")
sessionInfo()
sink()
## END
